from datetime import datetime, timedelta

from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ugettext
from django.utils.timezone import utc
from django.conf import settings

from .settings import DISCONNECTABLE_SIGNALS
from .exceptions import DependencyError


__all__ = [
    'Hider',
    'check_dependencies',
    'choicify',
    'get_key_by_value',
    'now',
    'now_after',
    'after',
]


class Hider(object):
    def __get__(self,instance,owner):
        raise AttributeError("Hidden attrbute")

    def __set__(self, obj, val):
        raise AttributeError("Hidden attribute")


def check_dependencies(dependencies, module):
    """
    Ensure dependencies of a module are listed in settings.INSTALLED_APPS
    
    :dependencies string | list: list of dependencies to check
    :module string: string representing the path to the current app
    """
    if type(dependencies) == str:
        dependencies = [dependencies]
    elif type(dependencies) != list:
        raise TypeError('dependencies argument must be of type list or string')
    
    for dependency in dependencies:
        if dependency not in settings.INSTALLED_APPS:
            raise DependencyError('%s depends on %s, which should be in settings.INSTALLED_APPS' % (module, dependency))


def choicify(dictionary):
    """
    Converts a readable python dictionary into a django model/form
    choice structure (list of tuples) ordered based on the values of each key
    
    :param dictionary: the dictionary to convert
    """
    # get order of the fields
    ordered_fields = sorted(dictionary, key=dictionary.get)
    choices = []
    # loop over each field
    for field in ordered_fields:
        # build tuple (value, i18n_key)
        row = (dictionary[field], _(field.replace('_', ' ')))
        # append tuple to choices
        choices.append(row)
    # return django sorted choices
    return choices


def get_key_by_value(dictionary, search_value):
    """
    searchs a value in a dicionary and returns the key of the first occurrence
    
    :param dictionary: dictionary to search in
    :param search_value: value to search for
    """
    for key, value in dictionary.iteritems():
        if value == search_value:
            return ugettext(key)


def pause_disconnectable_signals():
    """
    Disconnects non critical signals like notifications, websockets and stuff like that.
    Use when managing large chunks of nodes
    """
    for signal in DISCONNECTABLE_SIGNALS:
        signal['disconnect']()


def resume_disconnectable_signals():
    """
    Reconnects non critical signals like notifications, websockets and stuff like that.
    Use when managing large chunks of nodes
    """
    for signal in DISCONNECTABLE_SIGNALS:
        signal['reconnect']()


# time shortcuts

def now():
    """ returns the current date and time in UTC format (datetime object) """
    return datetime.utcnow().replace(tzinfo=utc)

def now_after(**kwargs):
    """ returns the current date and time plus the time (seconds, minutes, hours, days, years) specified """
    return now() + timedelta(**kwargs)

def ago(**kwargs):
    """ returns the current date and time minus the time (seconds, minutes, hours, days, years) specified """
    return now() - timedelta(**kwargs)

def after(date, **kwargs):
    """
    returns the result of the calculation of the date param plus the time (seconds, minutes, hours, days, years) specified
    
    :paramm datetime: datetime object to which add more time
    """
    return date + timedelta(**kwargs)



from django.core.exceptions import ObjectDoesNotExist
from django.http import Http404
from django.shortcuts import render_to_response, get_object_or_404, redirect
from django.template import RequestContext
from django.utils.http import base36_to_int
from django.utils.translation import ugettext
from django.contrib import messages
from django.contrib.auth.tokens import default_token_generator
from django.contrib.auth import get_user_model
User = get_user_model()

from .forms import ResetPasswordKeyForm
from .settings import settings, EMAIL_CONFIRMATION


def group_and_bridge(kwargs):
    """
    Given kwargs from the view (with view specific keys popped) pull out the
    bridge and fetch group from database.
    """

    bridge = kwargs.pop("bridge", None)

    if bridge:
        try:
            group = bridge.get_group(**kwargs)
        except ObjectDoesNotExist:
            raise Http404
    else:
        group = None

    return group, bridge


def group_context(group, bridge):
    # @@@ use bridge
    return {
        "group": group,
    }


def password_reset_from_key(request, uidb36, key, **kwargs):
    form_class = kwargs.get("form_class", ResetPasswordKeyForm)
    template_name = kwargs.get("template_name", "profiles/password_reset_from_key.html")
    token_generator = kwargs.get("token_generator", default_token_generator)

    group, bridge = group_and_bridge(kwargs)
    ctx = group_context(group, bridge)

    # pull out user
    try:
        uid_int = base36_to_int(uidb36)
    except ValueError:
        raise Http404

    user = get_object_or_404(User, id=uid_int)

    if token_generator.check_token(user, key):
        if request.method == "POST":
            password_reset_key_form = form_class(request.POST, user=user, temp_key=key)
            if password_reset_key_form.is_valid():
                password_reset_key_form.save()
                messages.add_message(
                    request,
                    messages.SUCCESS,
                    ugettext(u"Password successfully changed.")
                )
                password_reset_key_form = None
        else:
            password_reset_key_form = form_class()
        ctx.update({
            "form": password_reset_key_form,
        })
    else:
        ctx.update({
            "token_fail": True,
        })

    return render_to_response(template_name, RequestContext(request, ctx))


if EMAIL_CONFIRMATION:
    from .models import EmailConfirmation, EmailAddress
    from django.contrib.auth import login

    def confirm_email(request, confirmation_key):
        """ confirm email view """
        confirmation_key = confirmation_key.lower()
        # get email confirmation or 404
        email_confirmation = get_object_or_404(EmailConfirmation, key=confirmation_key)
        # make primary if no other primary addresses for this user
        make_primary = EmailAddress.objects.filter(
            user_id=email_confirmation.email_address.user_id,
            primary=True
        ).count() <= 0
        # confirm email
        email_address = EmailConfirmation.objects.confirm_email(
            confirmation_key,
            make_primary=make_primary
        )
        # log in the user if not already authenticated
        if email_address and not request.user.is_authenticated():
            user = email_address.user
            user.backend = 'django.contrib.auth.backends.ModelBackend'
            login(request, user)
        return redirect(settings.SITE_URL)



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Page'
        db.create_table(u'cms_page', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('added', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime(2014, 6, 3, 0, 0))),
            ('updated', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime(2014, 6, 3, 0, 0))),
            ('access_level', self.gf('django.db.models.fields.SmallIntegerField')(default=0)),
            ('title', self.gf('django.db.models.fields.CharField')(max_length=50)),
            ('slug', self.gf('django.db.models.fields.SlugField')(unique=True, max_length=50, blank=True)),
            ('content', self.gf('django.db.models.fields.TextField')()),
            ('is_published', self.gf('django.db.models.fields.BooleanField')(default=True)),
            ('meta_description', self.gf('django.db.models.fields.CharField')(max_length=255, blank=True)),
            ('meta_keywords', self.gf('django.db.models.fields.CharField')(max_length=255, blank=True)),
            ('meta_robots', self.gf('django.db.models.fields.CharField')(default='index, follow', max_length=50)),
        ))
        db.send_create_signal(u'cms', ['Page'])

        # Adding model 'MenuItem'
        db.create_table(u'cms_menuitem', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('added', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime(2014, 6, 3, 0, 0))),
            ('updated', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime(2014, 6, 3, 0, 0))),
            ('access_level', self.gf('django.db.models.fields.SmallIntegerField')(default=0)),
            ('order', self.gf('django.db.models.fields.PositiveIntegerField')(blank=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=50)),
            ('url', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('is_published', self.gf('django.db.models.fields.BooleanField')(default=True)),
        ))
        db.send_create_signal(u'cms', ['MenuItem'])


    def backwards(self, orm):
        # Deleting model 'Page'
        db.delete_table(u'cms_page')

        # Deleting model 'MenuItem'
        db.delete_table(u'cms_menuitem')


    models = {
        u'cms.menuitem': {
            'Meta': {'ordering': "['order']", 'object_name': 'MenuItem'},
            'access_level': ('django.db.models.fields.SmallIntegerField', [], {'default': '0'}),
            'added': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2014, 6, 3, 0, 0)'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_published': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'order': ('django.db.models.fields.PositiveIntegerField', [], {'blank': 'True'}),
            'updated': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2014, 6, 3, 0, 0)'}),
            'url': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        },
        u'cms.page': {
            'Meta': {'object_name': 'Page'},
            'access_level': ('django.db.models.fields.SmallIntegerField', [], {'default': '0'}),
            'added': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2014, 6, 3, 0, 0)'}),
            'content': ('django.db.models.fields.TextField', [], {}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_published': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'meta_description': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'meta_keywords': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'meta_robots': ('django.db.models.fields.CharField', [], {'default': "'index, follow'", 'max_length': '50'}),
            'slug': ('django.db.models.fields.SlugField', [], {'unique': 'True', 'max_length': '50', 'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'updated': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2014, 6, 3, 0, 0)'})
        }
    }

    complete_apps = ['cms']


from __future__ import absolute_import

from django.contrib.gis.geos import Point
from .base import XMLParserMixin, GenericGisSynchronizer


class OpenWisp(XMLParserMixin, GenericGisSynchronizer):
    """ OpenWisp GeoRSS synchronizer class """

    def parse(self):
        """ parse data """
        super(OpenWisp, self).parse()
        self.parsed_data = self.parsed_data.getElementsByTagName('item')

    def parse_item(self, item):
        guid = self.get_text(item, 'guid')
        name, created_at = guid.split('201', 1)  # ugly hack
        name = name.replace('_', ' ')
        created_at = "201%s" % created_at
        updated_at = self.get_text(item, 'updated')
        description = self.get_text(item, 'title')
        address = self.get_text(item, 'description')

        try:
            lat, lng = self.get_text(item, 'georss:point').split(' ')
        except IndexError:
            # detail view
            lat = self.get_text(item, 'georss:lat')
            lng = self.get_text(item, 'georss:long')

        # point object
        geometry = Point(float(lng), float(lat))

        result = {
            "name": name,
            "status": None,
            "address": address,
            "is_published": True,
            "user": None,
            "geometry": geometry,
            "elev": None,
            "description": description,
            "notes": guid,
            "added": created_at,
            "updated": updated_at,
            "data": {}
        }

        return result



from django.utils.translation import ugettext_lazy as _
from rest_framework import serializers
from rest_framework_gis import serializers as geoserializers
from rest_framework_hstore.serializers import HStoreSerializer
from nodeshot.core.base.serializers import DynamicRelationshipsMixin


class ExtensibleNodeSerializer(DynamicRelationshipsMixin, geoserializers.GeoModelSerializer, HStoreSerializer):
    """ node detail """
    user = serializers.Field(source='user.username')
    status = serializers.Field(source='status.slug')
    geometry = geoserializers.GeometryField(label=_('coordinates'))
    layer_name = serializers.Field(source='layer.name')
    access_level = serializers.Field(source='get_access_level_display')
    relationships = serializers.SerializerMethodField('get_relationships')
    # this is needed to avoid adding stuff to DynamicRelationshipsMixin
    _relationships = {}



import sys
import string
import random
import traceback
from optparse import make_option

from netaddr import ip

from django.core.management.base import BaseCommand
from django.db import transaction
from django.db.models import Q
from django.core.exceptions import ImproperlyConfigured
from django.contrib.gis.geos import Point
from django.utils.text import slugify
from django.contrib.contenttypes.models import ContentType
from django.contrib.auth import get_user_model
User = get_user_model()

from ...settings import STATUS_MAPPING, DEFAULT_LAYER

from nodeshot.community.profiles.settings import EMAIL_CONFIRMATION

if EMAIL_CONFIRMATION:
    from nodeshot.community.profiles.models import EmailAddress

from nodeshot.core.base.utils import pause_disconnectable_signals, resume_disconnectable_signals
from nodeshot.core.nodes.models import Node, Status
from nodeshot.networking.net.models import *  # noqa
from nodeshot.networking.net.models.choices import INTERFACE_TYPES
from nodeshot.networking.links.models import Link
from nodeshot.networking.links.models.choices import LINK_STATUS, LINK_TYPES
from nodeshot.community.mailing.models import Inward
from nodeshot.interop.oldimporter.models import *  # noqa


class Command(BaseCommand):
    """
    Will try to import data from old nodeshot.

    Requirements for settings:
        * nodeshot.interop.oldimporter must be in INSTALLED_APPS
        * old_nodeshot database must be configured
        * database routers directives must be uncommented

    Steps:

    1.  Retrieve all nodes
        Retrieve all nodes from old db and convert queryset in a python list.

    2.  Extract user data from nodes

        (Since in old nodeshot there are no users but each node contains data
        such as name, email, and stuff like that)

            * loop over nodes and extract a list of unique emails
            * each unique email will be a new user in the new database
            * each new user will have a random password set
            * save users, email addresses

    3.  Import nodes

            * USER: assign owner (the link is the email)
            * LAYER: assign layer (layers must be created by hand first!):
                1. if node has coordinates comprised in a specified layer choose that
                2. if node has coordinates comprised in more than one layer prompt the user which one to choose
                3. if node does not have coordinates comprised in any layer:
                    1. use default layer if specified (configured in settings.NODESHOT_OLDIMPORTER_DEFAULT_LAYER)
                    2. discard the node if no default layer specified
            * STATUS: assign status depending on configuration:
                settings.NODESHOT_OLDIMPORTER_STATUS_MAPPING must be a dictionary in which the
                key is the old status value while the value is the new status value
                if settings.NODESHOT_OLDIMPORTER_STATUS_MAPPING is False the default status will be used
            * HOSTPOT: if status is hotspot or active and hotspot add this info in HSTORE data field

    4.  Import devices
        Create any missing routing protocol

    5.  Import interfaces, ip addresses, vaps

    6.  Import links

    7.  Import Contacts

    TODO: Decide what to do with statistics and hna.
    """
    help = 'Import old nodeshot data. Layers and Status must be created first.'

    status_mapping = STATUS_MAPPING
    # if no default layer some nodes might be discarded
    default_layer = DEFAULT_LAYER

    old_nodes = []
    saved_users = []
    saved_nodes = []
    saved_devices = []
    routing_protocols_added = []
    saved_interfaces = []
    saved_vaps = []
    saved_ipv4 = []
    saved_ipv6 = []
    saved_links = []
    saved_contacts = []

    option_list = BaseCommand.option_list + (
        make_option(
            '--noinput',
            action='store_true',
            dest='noinput',
            default=False,
            help='Do not prompt for user intervention and use default settings'
        ),
        make_option(
            '--nodelete',
            action='store_true',
            dest='nodelete',
            default=False,
            help='Do not delete imported data if any uncaught exception occurs'
        ),
    )

    def message(self, message):
        self.stdout.write('%s\n\r' % message)

    def verbose(self, message):
        if self.verbosity == 2:
            self.message(message)

    def handle(self, *args, **options):
        """ execute synchronize command """
        self.options = options
        delete = False

        try:
            # blank line
            self.stdout.write('\r\n')
            # store verbosity level in instance attribute for later use
            self.verbosity = int(self.options.get('verbosity'))

            self.verbose('disabling signals (notififcations, websocket alerts)')
            pause_disconnectable_signals()

            self.check_status_mapping()
            self.retrieve_nodes()
            self.extract_users()
            self.import_admins()
            self.import_users()
            self.import_nodes()
            self.import_devices()
            self.import_interfaces()
            self.import_links()
            self.import_contacts()

            self.confirm_operation_completed()

            resume_disconnectable_signals()
            self.verbose('re-enabling signals (notififcations, websocket alerts)')

        except KeyboardInterrupt:
            self.message('\n\nOperation cancelled...')
            delete = True
        except Exception:
            tb = traceback.format_exc()
            delete = True
            # rollback database transaction
            transaction.rollback()
            self.message('Got exception:\n\n%s' % tb)

        if delete:
            self.delete_imported_data()

    def confirm_operation_completed(self):
        # if noinput param do not ask for confirmatin
        if self.options.get('noinput') is True:
            return

        self.message("Are you satisfied with the results? If not all imported data will be deleted\n\n[Y/n]")

        while True:
            answer = raw_input().lower()
            if answer == '':
                answer = "y"

            if answer in ['y', 'n']:
                break
            else:
                self.message("Please respond with one of the valid answers\n")

        if answer == 'n':
            self.delete_imported_data()
        else:
            self.message('Operation completed!')

    def delete_imported_data(self):
        if self.options.get('nodelete') is True:
            self.message('--nodelete option specified, won\'t delete the imported data')
            return

        self.message('Going to delete all the imported data...')

        for interface in self.saved_interfaces:
            try:
                interface.delete()
            except Exception:
                tb = traceback.format_exc()
                self.message('Got exception while deleting interface %s\n\n%s' % (interface.mac, tb))

        for device in self.saved_devices:
            try:
                device.delete()
            except Exception:
                tb = traceback.format_exc()
                self.message('Got exception while deleting device %s\n\n%s' % (device.name, tb))

        for routing_protocol in self.routing_protocols_added:
            try:
                routing_protocol.delete()
            except Exception:
                tb = traceback.format_exc()
                self.message('Got exception while deleting routing_protocol %s\n\n%s' % (routing_protocol.name, tb))

        for node in self.saved_nodes:
            try:
                node.delete()
            except Exception:
                tb = traceback.format_exc()
                self.message('Got exception while deleting node %s\n\n%s' % (node.name, tb))

        for contact in self.saved_contacts:
            try:
                contact.delete()
            except Exception:
                tb = traceback.format_exc()
                self.message('Got exception while deleting contact log entry %s\n\n%s' % (contact.id, tb))

        for user in self.saved_users:
            try:
                user.delete()
            except Exception:
                tb = traceback.format_exc()
                self.message('Got exception while deleting user %s\n\n%s' % (user.username, tb))

    def prompt_layer_selection(self, node, layers):
        """Ask user what to do when an old node is contained in more than one layer.
        Possible answers are:
            * use default layer (default answer if pressing enter)
            * choose layer
            * discard node
        """
        valid = {
            "default": "default",
            "def": "default",
            "discard": "discard",
            "dis": "discard",
        }
        question = """Cannot automatically determine layer for node "%s" because there \
are %d layers available in that area, what do you want to do?\n\n""" % (node.name, len(layers))

        available_layers = ""
        for layer in layers:
            available_layers += "%d (%s)\n" % (layer.id, layer.name)
            valid[str(layer.id)] = layer.id

        prompt = """\
choose (enter the number of) one of the following layers:
%s
"default"    use default layer (if no default layer specified in settings node will be discarded)
"discard"    discard node

(default action is to use default layer)\n\n""" % available_layers

        sys.stdout.write(question + prompt)

        while True:
            if self.options.get('noinput') is True:
                answer = 'default'
                break

            answer = raw_input().lower()
            if answer == '':
                answer = "default"

            if answer in valid:
                answer = valid[answer]
                break
            else:
                sys.stdout.write("Please respond with one of the valid answers\n")

        sys.stdout.write("\n")
        return answer

    @classmethod
    def generate_random_password(cls, size=10, chars=string.ascii_uppercase + string.digits):
        return ''.join(random.choice(chars) for x in range(size))

    def check_status_mapping(self):
        """ ensure status map does not contain status values which are not present in DB """
        self.verbose('checking status mapping...')

        if not self.status_mapping:
            self.message('no status mapping found')
            return

        for old_val, new_val in self.status_mapping.iteritems():
            try:
                # look up by slug if new_val is string
                if isinstance(new_val, basestring):
                    lookup = {'slug': new_val}
                # lookup by primary key otherwise
                else:
                    lookup = {'pk': new_val}
                status = Status.objects.get(**lookup)
                self.status_mapping[old_val] = status.id
            except Status.DoesNotExist:
                raise ImproperlyConfigured('Error! Status with slug %s not found in the database' % new_val)

        self.verbose('status map correct')

    def get_status(self, value):
        return self.status_mapping.get(value, self.status_mapping['default'])

    def retrieve_nodes(self):
        """ retrieve nodes from old mysql DB """
        self.verbose('retrieving nodes from old mysql DB...')

        self.old_nodes = list(OldNode.objects.all())
        self.message('retrieved %d nodes' % len(self.old_nodes))

    def extract_users(self):
        """ extract user info """
        email_set = set()
        users_dict = {}

        self.verbose('going to extract user information from retrieved nodes...')

        for node in self.old_nodes:
            email_set.add(node.email)

            if node.email not in users_dict:
                users_dict[node.email] = {
                    'owner': node.owner
                }

        self.email_set = email_set
        self.users_dict = users_dict

        self.verbose('%d users extracted' % len(email_set))

    def import_admins(self):
        """ save admins to local DB """
        self.message('saving admins into local DB')

        saved_admins = []

        for olduser in OldUser.objects.all():
            try:
                user = User.objects.get(username=olduser.username)
            except User.DoesNotExist:
                user = User()
            user.username = olduser.username
            user.password = olduser.password
            user.first_name = olduser.first_name
            user.last_name = olduser.last_name
            user.email = olduser.email
            user.is_active = olduser.is_active
            user.is_staff = olduser.is_staff
            user.is_superuser = olduser.is_superuser
            user.date_joined = olduser.date_joined
            user.full_clean()
            user.save(sync_emailaddress=False)
            saved_admins.append(user)

            # mark email address as confirmed if feature is enabled
            if EMAIL_CONFIRMATION and EmailAddress.objects.filter(email=user.email).count() is 0:
                try:
                    email_address = EmailAddress(user=user, email=user.email, verified=True, primary=True)
                    email_address.full_clean()
                    email_address.save()
                except Exception:
                    tb = traceback.format_exc()
                    self.message('Could not save email address for user %s, got exception:\n\n%s' % (user.username, tb))

        self.message('saved %d admins into local DB' % len(saved_admins))
        self.saved_admins = saved_admins

    def import_users(self):
        """ save users to local DB """
        self.message('saving users into local DB')

        saved_users = self.saved_admins

        # loop over all extracted unique email addresses
        for email in self.email_set:
            owner = self.users_dict[email].get('owner')

            # if owner is not specified, build username from email
            if owner.strip() == '':
                owner, domain = email.split('@')
                # replace any points with a space
                owner = owner.replace('.', ' ')

            # if owner has a space, assume he specified first and last name
            if ' ' in owner:
                owner_parts = owner.split(' ')
                first_name = owner_parts[0]
                last_name = owner_parts[1]
            else:
                first_name = owner
                last_name = ''

            # username must be slugified otherwise won't get into the DB
            username = slugify(owner)

            # check if user exists first
            try:
                # try looking by email
                user = User.objects.get(email=email)
            except User.DoesNotExist:
                # otherwise init new
                user = User()
                user.username = username
                # generate new password only for new users
                user.password = self.generate_random_password()
                user.is_active = True

            # we'll create one user for each unique email address we've got
            user.first_name = first_name.capitalize()
            user.last_name = last_name.capitalize()
            user.email = email

            # extract date joined from old nodes
            # find the oldest node of this user
            oldest_node = OldNode.objects.filter(email=email).order_by('added')[0]
            user.date_joined = oldest_node.added

            # be sure username is unique
            counter = 1
            original_username = username
            while True:
                # do this check only if user is new
                if not user.pk and User.objects.filter(username=user.username).count() > 0:
                    counter += 1
                    user.username = '%s%d' % (original_username, counter)
                else:
                    break

            try:
                # validate data and save
                user.full_clean()
                user.save(sync_emailaddress=False)
            except Exception:
                # if user already exists use that instance
                if(User.objects.filter(email=email).count() == 1):
                    user = User.objects.get(email=email)
                # otherwise report error
                else:
                    user = None
                    tb = traceback.format_exc()
                    self.message('Could not save user %s, got exception:\n\n%s' % (user.username, tb))

            # if we got a user to add
            if user is not None:
                # store id
                self.users_dict[email]['id'] = user.id
                # append to saved users
                saved_users.append(user)
                self.verbose('Saved user %s (%s) with email <%s>' % (user.username, user.get_full_name(), user.email))

            # mark email address as confirmed if feature is enabled
            if EMAIL_CONFIRMATION and EmailAddress.objects.filter(email=user.email).count() is 0:
                try:
                    email_address = EmailAddress(user=user, email=user.email, verified=True, primary=True)
                    email_address.full_clean()
                    email_address.save()
                except Exception:
                    tb = traceback.format_exc()
                    self.message('Could not save email address for user %s, got exception:\n\n%s' % (user.username, tb))

        self.message('saved %d users into local DB' % len(saved_users))
        self.saved_users = saved_users

    def import_nodes(self):
        """ import nodes into local DB """
        self.message('saving nodes into local DB...')

        saved_nodes = []

        # loop over all old node and create new nodes
        for old_node in self.old_nodes:
            # if this old node is unconfirmed skip to next cycle
            if old_node.status == 'u':
                continue

            try:
                node = Node.objects.get(pk=old_node.id)
            except Node.DoesNotExist:
                node = Node(id=old_node.id)
                node.data = {}

            node.user_id = self.users_dict[old_node.email]['id']
            node.name = old_node.name
            node.slug = old_node.slug
            node.geometry = Point(old_node.lng, old_node.lat)
            node.elev = old_node.alt
            node.description = old_node.description
            node.notes = old_node.notes
            node.added = old_node.added
            node.updated = old_node.updated

            intersecting_layers = node.intersecting_layers
            # if more than one intersecting layer
            if len(intersecting_layers) > 1:
                # prompt user
                answer = self.prompt_layer_selection(node, intersecting_layers)
                if isinstance(answer, int):
                    node.layer_id = answer
                elif answer == 'default' and self.default_layer is not False:
                    node.layer_id = self.default_layer
                else:
                    self.message('Node %s discarded' % node.name)
                    continue
            # if one intersecting layer select that
            elif 2 > len(intersecting_layers) > 0:
                node.layer = intersecting_layers[0]
            # if no intersecting layers
            else:
                if self.default_layer is False:
                    # discard node if no default layer specified
                    self.message("""Node %s discarded because is not contained
                                 in any specified layer and no default layer specified""" % node.name)
                    continue
                else:
                    node.layer_id = self.default_layer

            if old_node.postal_code:
                # additional info
                node.data['postal_code'] = old_node.postal_code

            # is it a hotspot?
            if old_node.status in ['h', 'ah']:
                node.data['is_hotspot'] = 'true'

            # determine status according to settings
            if self.status_mapping:
                node.status_id = self.get_status(old_node.status)

            try:
                node.full_clean()
                node.save(auto_update=False)
                saved_nodes.append(node)
                self.verbose('Saved node %s in layer %s with status %s' % (node.name, node.layer, node.status.name))
            except Exception:
                tb = traceback.format_exc()
                self.message('Could not save node %s, got exception:\n\n%s' % (node.name, tb))

        self.message('saved %d nodes into local DB' % len(saved_nodes))
        self.saved_nodes = saved_nodes

    def import_devices(self):
        self.verbose('retrieving devices from old mysql DB...')
        self.old_devices = list(OldDevice.objects.all())
        self.message('retrieved %d devices' % len(self.old_devices))

        saved_devices = []
        routing_protocols_added = []

        for old_device in self.old_devices:
            try:
                device = Device.objects.get(pk=old_device.id,)
            except Device.DoesNotExist:
                device = Device(id=old_device.id)

            device.node_id = old_device.node_id
            device.type = "radio"
            device.name = old_device.name
            device.description = old_device.description
            device.added = old_device.added
            device.updated = old_device.updated
            device.data = {
                "model": old_device.type,
                "cname": old_device.cname
            }

            try:
                device.full_clean()
                device.save(auto_update=False)
                saved_devices.append(device)
                self.verbose('Saved device %s' % device.name)
            except Exception:
                tb = traceback.format_exc()
                self.message('Could not save device %s, got exception:\n\n%s' % (device.name, tb))

            try:
                routing_protocol = RoutingProtocol.objects.filter(name__icontains=old_device.routing_protocol)[0]
            except IndexError:
                routing_protocol = RoutingProtocol.objects.create(name=old_device.routing_protocol)
                routing_protocols_added.append(routing_protocol)
            device.routing_protocols.add(routing_protocol)

        self.message('saved %d devices into local DB' % len(saved_devices))
        self.saved_devices = saved_devices
        self.routing_protocols_added = routing_protocols_added

    def import_interfaces(self):
        self.verbose('retrieving interfaces from old mysql DB...')
        self.old_interfaces = list(OldInterface.objects.all())
        self.message('retrieved %d interfaces' % len(self.old_interfaces))

        saved_interfaces = []
        saved_vaps = []
        saved_ipv4 = []
        saved_ipv6 = []

        for old_interface in self.old_interfaces:
            interface_dict = {
                "id": old_interface.id,
                "device_id": int(old_interface.device_id),
                "mac": old_interface.mac_address,
                "name": old_interface.cname[0:10],
                "added": old_interface.added,
                "updated": old_interface.updated,
                "data": {}
            }
            vap = None
            ipv4 = None
            ipv6 = None

            # determine interface type and specific fields
            if old_interface.type == 'eth':
                interface_dict['standard'] = 'fast'
                interface_dict['duplex'] = 'full'
                InterfaceModel = Ethernet
            elif old_interface.type == 'wifi':
                interface_dict['mode'] = old_interface.wireless_mode
                interface_dict['channel'] = old_interface.wireless_channel
                InterfaceModel = Wireless
                # determine ssid
                if old_interface.essid or old_interface.bssid:
                    vap = Vap(**{
                        "interface_id": old_interface.id,
                        "essid": old_interface.essid,
                        "bssid": old_interface.bssid
                    })
                    # if vap already exists flag it for UPDATE instead of INSERT
                    try:
                        v = Vap.objects.get(
                            Q(interface_id=old_interface.id) & (
                                Q(essid=old_interface.essid) |
                                Q(bssid=old_interface.bssid)
                            )
                        )
                        # trick to make django do an update query instead of an insert
                        # working on django 1.6
                        vap.id = v.id
                        vap._state.adding = False
                    except Vap.DoesNotExist:
                        pass
                if old_interface.essid:
                    interface_dict['data']['essid'] = old_interface.essid
                if old_interface.bssid:
                    interface_dict['data']['bssid'] = old_interface.bssid
            elif old_interface.type == 'bridge':
                InterfaceModel = Bridge
            elif old_interface.type == 'vpn':
                InterfaceModel = Tunnel
            else:
                interface_dict['type'] = INTERFACE_TYPES.get('virtual')
                interface_dict['data']['old_nodeshot_interface_type'] = old_interface.get_type_display()
                InterfaceModel = Interface

            interface = InterfaceModel(**interface_dict)
            # if interface already exists flag it for UPDATE instead of INSERT
            try:
                InterfaceModel.objects.get(pk=old_interface.id)
                interface._state.adding = False
            except InterfaceModel.DoesNotExist:
                pass

            if old_interface.ipv4_address:
                old_interface.ipv4_address = old_interface.ipv4_address.strip()  # stupid django bug
                ipv4 = Ip(**{
                    "interface_id": old_interface.id,
                    "address": old_interface.ipv4_address
                })
                # if ip already exists flag it for UPDATE instead of INSERT
                try:
                    ipv4.id = Ip.objects.get(address=old_interface.ipv4_address).id
                    ipv4._state.adding = False
                except Ip.DoesNotExist:
                    pass
                # ensure ipv4 is valid
                try:
                    ip.IPAddress(old_interface.ipv4_address)
                except (ip.AddrFormatError, ValueError):
                    self.message('Invalid IPv4 address %s' % (old_interface.ipv4_address))
                    ipv4 = None

            if old_interface.ipv6_address:
                old_interface.ipv6_address = old_interface.ipv6_address.strip()  # stupid django bug
                ipv6 = Ip(**{
                    "interface_id": old_interface.id,
                    "address": old_interface.ipv6_address
                })
                # if ip already exists flag it for UPDATE instead of INSERT
                try:
                    ipv6.id = Ip.objects.get(address=old_interface.ipv6_address).id
                    ipv6._state.adding = False
                except Ip.DoesNotExist:
                    pass
                # ensure ipv6 is valid
                try:
                    ip.IPAddress(old_interface.ipv6_address)
                except (ip.AddrFormatError, ValueError):
                    self.message('Invalid IPv6 address %s' % (old_interface.ipv6_address))
                    ipv6 = None

            try:
                interface.full_clean()
                interface.save(auto_update=False)
                saved_interfaces.append(interface)
                self.verbose('Saved interface %s' % interface.name)
            except Exception:
                tb = traceback.format_exc()
                self.message('Could not save interface %s, got exception:\n\n%s' % (interface.mac, tb))
                continue

            if vap:
                try:
                    vap.full_clean()
                    vap.save()
                    saved_vaps.append(vap)
                    self.verbose('Saved vap %s' % vap.essid or vap.bssid)
                except Exception:
                    tb = traceback.format_exc()
                    self.message('Could not save vap %s, got exception:\n\n%s' % (vap.essid or vap.bssid, tb))

            if ipv4:
                try:
                    ipv4.full_clean()
                    ipv4.save()
                    saved_ipv4.append(ipv4)
                    self.verbose('Saved ipv4 %s' % ipv4.address)
                except Exception:
                    tb = traceback.format_exc()
                    self.message('Could not save ipv4 %s, got exception:\n\n%s' % (ipv4.address, tb))

            if ipv6:
                try:
                    ipv6.full_clean()
                    ipv6.save()
                    saved_ipv6.append(ipv6)
                    self.verbose('Saved ipv6 %s' % ipv6.address)
                except Exception:
                    tb = traceback.format_exc()
                    self.message('Could not save ipv6 %s, got exception:\n\n%s' % (ipv6.address, tb))

        self.message('saved %d interfaces into local DB' % len(saved_interfaces))
        self.message('saved %d vaps into local DB' % len(saved_vaps))
        self.message('saved %d ipv4 addresses into local DB' % len(saved_ipv4))
        self.message('saved %d ipv6 addresses into local DB' % len(saved_ipv6))
        self.saved_interfaces = saved_interfaces
        self.saved_vaps = saved_vaps
        self.saved_ipv4 = saved_ipv4
        self.saved_ipv6 = saved_ipv6

    def import_links(self):
        self.verbose('retrieving links from old mysql DB...')
        self.old_links = list(OldLink.objects.all())
        self.message('retrieved %d links' % len(self.old_links))

        saved_links = []

        for old_link in self.old_links:

            skip = False

            try:
                interface_a = Interface.objects.get(pk=old_link.from_interface_id)
                if interface_a.type != INTERFACE_TYPES.get('wireless'):
                    interface_a.type = INTERFACE_TYPES.get('wireless')
                    interface_a.save()
            except Interface.DoesNotExist:
                self.message('Interface #%s does not exist, probably link #%s is orphan!' % (old_link.from_interface_id, old_link.id))
                skip = True

            try:
                interface_b = Interface.objects.get(pk=old_link.to_interface_id)
                if interface_b.type != INTERFACE_TYPES.get('wireless'):
                    interface_b.type = INTERFACE_TYPES.get('wireless')
                    interface_b.save()
            except Interface.DoesNotExist:
                self.message('Interface #%s does not exist, probably link #%s is orphan!' % (old_link.to_interface_id, old_link.id))
                skip = True

            if skip:
                self.verbose('Skipping to next cycle')
                continue

            old_bandwidth = [old_link.sync_tx, old_link.sync_rx]

            link = Link(**{
                "id": old_link.id,
                "interface_a": interface_a,
                "interface_b": interface_b,
                "status": LINK_STATUS.get('active'),
                "type": LINK_TYPES.get('radio'),
                "metric_type": 'etx',
                "metric_value": old_link.etx,
                "dbm": old_link.dbm,
                "min_rate": min(old_bandwidth),
                "max_rate": max(old_bandwidth),
            })
            # if link already exists flag it for UPDATE instead of INSERT
            try:
                Link.objects.get(pk=old_link.id)
                link._state.adding = False
            except Link.DoesNotExist:
                pass

            if old_link.hide:
                link.access_level = 3

            try:
                link.full_clean()
                link.save()
                saved_links.append(link)
                self.verbose('Saved link %s' % link)
            except Exception:
                tb = traceback.format_exc()
                self.message('Could not save link %s, got exception:\n\n%s' % (old_link.id, tb))

        self.message('saved %d links into local DB' % len(saved_links))
        self.saved_links = saved_links

    def import_contacts(self):
        self.verbose('retrieving contact log from old mysql DB...')
        self.old_contacts = list(OldContact.objects.all())
        self.message('retrieved %d entries from contact log' % len(self.old_contacts))

        saved_contacts = []

        content_type = ContentType.objects.only('id', 'model').get(app_label='nodes', model='node')

        for old_contact in self.old_contacts:

            try:
                user = User.objects.get(email=old_contact.from_email)
            except User.DoesNotExist:
                user = None
                self.message('Could not find any user with email: %s' % old_contact.from_email)

            contact = Inward(**{
                "id": old_contact.id,
                "content_type": content_type,
                "object_id": old_contact.node_id,
                "status": 1,  # sent,
                "from_name": old_contact.from_name,
                "from_email": old_contact.from_email,
                "message": old_contact.message,
                "ip": old_contact.ip,
                "user_agent": old_contact.user_agent,
                "accept_language": old_contact.accept_language,
                "added": old_contact.date,
                "updated": old_contact.date,
            })
            if user:
                contact.user = user

            # if contact already exists flag it for UPDATE instead of INSERT
            try:
                Inward.objects.get(pk=old_contact.id)
                contact._state.adding = False
            except Inward.DoesNotExist:
                pass

            try:
                contact.full_clean(exclude=['user'])
                contact.save(auto_update=False)
                saved_contacts.append(contact)
                self.verbose('Saved contact log entry #%s' % contact.id)
            except Exception:
                tb = traceback.format_exc()
                self.message('Could not save contact log entry %s, got exception:\n\n%s' % (old_contact.id, tb))

        self.message('saved %d entries of contact log into local DB' % len(saved_contacts))
        self.saved_contacts = saved_contacts



from celery import task


# ------ Asynchronous tasks ------ #


@task
def send_outward_mails(queryset):
    """
    Call send method on Outward objects in the background
    """
    for obj in queryset:
        obj.send()



from nodeshot.core.base.utils import check_dependencies

check_dependencies(
    dependencies='nodeshot.community.profiles',
    module='nodeshot.community.notifications'
)


from .notification import Notification
from .user_settings import UserEmailNotificationSettings, UserWebNotificationSettings
from ..settings import settings, REGISTER


__all__ = [
    'Notification',
    'UserWebNotificationSettings',
    'UserEmailNotificationSettings'
]


from django.dispatch import receiver
from django.db.models.signals import post_save
from nodeshot.community.profiles.models import Profile


if 'nodeshot.community.notifications' in settings.INSTALLED_APPS:
    @receiver(post_save, sender=Profile)
    def create_settings(sender, **kwargs):
        """ create user notification settings on user creation """
        created = kwargs['created']
        user = kwargs['instance']
        if created:
            UserWebNotificationSettings.objects.create(user=user)
            UserEmailNotificationSettings.objects.create(user=user)

    # ------ register notification signals ------ #

    from importlib import import_module

    for module in REGISTER:
        import_module(module)



from django.utils import timezone
from django.contrib.auth.models import UserManager


class ProfileManager(UserManager):
    """
    adds support for sync_emailaddress parameter
    """
    def _create_user(self, username, email, password, is_staff,
                     is_superuser, sync_emailaddress=True, **extra_fields):
        """
        Creates and saves a User with the given username, email and password.
        """
        now = timezone.now()
        if not username:
            raise ValueError('The given username must be set')
        email = self.normalize_email(email)
        user = self.model(username=username, email=email,
                          is_staff=is_staff, is_active=True,
                          is_superuser=is_superuser, last_login=now,
                          date_joined=now, **extra_fields)
        user.set_password(password)
        user.save(using=self._db, sync_emailaddress=sync_emailaddress)
        return user

    def create_user(self, username, email=None, password=None,
                    sync_emailaddress=True, **extra_fields):
        return self._create_user(username, email, password, False, False,
                                 sync_emailaddress, **extra_fields)



from django.db import models
from django.utils.translation import ugettext_lazy as _

from nodeshot.core.base.models import BaseDate
from . import ImageMixin


class Manufacturer(BaseDate, ImageMixin):
    """
    Manufacturer Model
    Eg: Ubiquiti, Mikrotic, Dlink, ecc.
    """
    name = models.CharField(_('name'), max_length=50, unique=True)
    url = models.URLField(_('url'), blank=True)
    image = models.ImageField(_('logo'), blank=True, upload_to='manufacturers/')
    image_width = 180
    
    class Meta:
        app_label= 'hardware'
        verbose_name = _('Manufacturer')
        verbose_name_plural = _('Manufacturers')
        ordering = ['name']
    
    def __unicode__(self):
        return self.name
    
    def url_tag(self):
        return '<a href="%s" target="_blank">%s</a>' % (self.url, self.url)
        
    url_tag.allow_tags = True



"""
nodeshot.core.layers unit tests
"""

import simplejson as json

from django.test import TestCase
from django.core.exceptions import ValidationError
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext as _
from django.contrib.gis.geos import GEOSGeometry, Point

from nodeshot.core.base.tests import user_fixtures
from nodeshot.core.nodes.models import Node  # test additional validation added by layer model

from .models import Layer


class LayerTest(TestCase):
    fixtures = [
        'initial_data.json',
        user_fixtures,
        'test_layers.json',
        'test_status.json',
        'test_nodes.json'
    ]

    def test_layer_manager(self):
        """ test Layer custom Manager """
        # published() method
        layers_count = Layer.objects.all().count()
        published_layers_count = Layer.objects.published().count()
        self.assertEquals(published_layers_count, layers_count)

        # after unpublishing one layer we should get 1 less layer in total
        l = Layer.objects.get(pk=1)
        l.is_published = False
        l.save()
        layers_count = Layer.objects.all().count()
        published_layers_count = Layer.objects.published().count()
        self.assertEquals(published_layers_count, layers_count - 1)

        # external() method
        self.assertEquals(Layer.objects.external().count(), Layer.objects.filter(is_external=True).count())

        # mix external and published
        count = Layer.objects.filter(is_external=True, is_published=True).count()
        self.assertEquals(Layer.objects.external().published().count(), count)
        self.assertEquals(Layer.objects.published().external().count(), count)

    def test_layer_new_nodes_allowed(self):
        layer = Layer.objects.get(pk=1)
        layer.new_nodes_allowed = False
        layer.full_clean()
        layer.save()

        # ensure changing an existing node works
        node = layer.node_set.all()[0]
        node.name = 'changed'
        node.full_clean()
        node.save()
        # re-get from DB, just to be sure
        node = Node.objects.get(pk=node.pk)
        self.assertEqual(node.name, 'changed')

        # ensure new node cannot be added
        node = Node(**{
            'name': 'test new node',
            'slug': 'test-new-node',
            'layer': layer,
            'geometry': 'POINT (10.4389188797003565 43.7200020000987328)'
        })
        with self.assertRaises(ValidationError):
            node.full_clean()

        try:
            node.full_clean()
            assert()
        except ValidationError as e:
            self.assertIn(_('New nodes are not allowed for this layer'), e.messages)

    def test_layer_nodes_minimum_distance(self):
        """ ensure minimum distance settings works as expected """
        layer = Layer.objects.get(slug='rome')
        node = layer.node_set.all()[0]

        # creating node with same coordinates should not be an issue
        new_node = Node(**{
            'name': 'new_node',
            'slug': 'new_node',
            'layer': layer,
            'geometry': node.geometry
        })
        new_node.full_clean()
        new_node.save()

        layer.nodes_minimum_distance = 100
        layer.save()

        try:
            new_node.full_clean()
        except ValidationError as e:
            self.assertIn(_('Distance between nodes cannot be less than %s meters') % layer.nodes_minimum_distance, e.messages)
            return

        self.assertTrue(False, 'validation not working as expected')

    def test_layers_api(self, *args, **kwargs):
        """
        Layers endpoint should be reachable and return 404 if layer is not found.
        """
        layer = Layer.objects.get(pk=1)
        layer_slug = layer.slug
        fake_layer_slug = "idontexist"

        # api_layer list
        response = self.client.get(reverse('api_layer_list'))
        self.assertEqual(response.status_code, 200)
        self.assertIn('"is_external": false', response.content)

        # api's expecting slug in request,test with existing and fake slug
        # api_layer_detail
        response = self.client.get(reverse('api_layer_detail', args=[layer_slug]))
        self.assertEqual(response.status_code, 200)
        response = self.client.get(reverse('api_layer_detail', args=[fake_layer_slug]))
        self.assertEqual(response.status_code, 404)

        # api_layer_nodes
        response = self.client.get(reverse('api_layer_nodes_list', args=[layer_slug]))
        self.assertEqual(response.status_code, 200)
        response = self.client.get(reverse('api_layer_nodes_list', args=[fake_layer_slug]))
        self.assertEqual(response.status_code, 404)

        # api_layer_nodes_geojson
        response = self.client.get(reverse('api_layer_nodes_geojson', args=[layer_slug]))
        self.assertEqual(response.status_code, 200)
        response = self.client.get(reverse('api_layer_nodes_geojson', args=[fake_layer_slug]))
        self.assertEqual(response.status_code, 404)

    def test_layers_api_results(self, *args, **kwargs):
        """
        layers resources should return the expected number of objects
        """
        layer = Layer.objects.get(pk=1)
        layer_count = Layer.objects.all().count()
        layer_slug = layer.slug

        # api_layer list
        response = self.client.get(reverse('api_layer_list'))
        api_layer_count = len(response.data)
        self.assertEqual(api_layer_count, layer_count)

        # api_layer_nodes_list
        response = self.client.get(reverse('api_layer_nodes_list', args=[layer_slug]))
        layer_public_nodes_count = Node.objects.filter(layer=layer).published().access_level_up_to('public').count()
        self.assertEqual(response.data['count'], layer_public_nodes_count)
        self.assertEqual(len(response.data['results']), layer_public_nodes_count)

        # api_layer_nodes_geojson
        response = self.client.get(reverse('api_layer_nodes_geojson', args=[layer_slug]))
        self.assertEqual(len(response.data['features']), layer_public_nodes_count)

    def test_layers_api_post(self):
        layer_count = Layer.objects.all().count()
        # POST to create, 400
        self.client.login(username='registered', password='tester')
        data = {
            "name": "test",
            "slug": "test",
            "center": "POINT (38.1154075128999921 12.5107643007999929)",
            "area": "POINT (38.1154075128999921 12.5107643007999929)"
        }
        response = self.client.post(reverse('api_layer_list'), json.dumps(data), content_type='application/json')
        self.assertEqual(response.status_code, 403)
        self.assertEqual(layer_count, Layer.objects.all().count())

        # POST to create 200
        self.client.logout()
        self.client.login(username='admin', password='tester')
        response = self.client.post(reverse('api_layer_list'), json.dumps(data), content_type='application/json')
        self.assertEqual(response.status_code, 201)
        self.assertEqual(Layer.objects.all().count(), layer_count + 1)

    def test_unpublish_layer_should_unpublish_nodes(self):
        layer = Layer.objects.first()
        layer.is_published = False
        layer.full_clean()
        layer.save()
        for node in layer.node_set.all():
            self.assertFalse(node.is_published)

        layer = Layer.objects.first()
        layer.is_published = True
        layer.full_clean()
        layer.save()
        for node in layer.node_set.all():
            self.assertTrue(node.is_published)

    def test_layer_area_point_or_polygon(self):
        layer = Layer.objects.get(slug='rome')
        layer.area = GEOSGeometry('LINESTRING (12.19 41.92, 12.58 42.17)')
        try:
            layer.full_clean()
        except ValidationError as e:
            self.assertIn('Polygon', str(e))
            self.assertIn('Point', str(e))
        else:
            self.fail('ValidationError not raised (Polygon/Point area check)')
        layer.area = GEOSGeometry('POINT (12.19 41.92)')
        layer.full_clean()

    def test_layer_area_contains_node_validation(self):
        """ ensure area validation works as expected """
        layer = Layer.objects.get(slug='rome')
        layer.area = GEOSGeometry('POLYGON ((12.19 41.92, 12.58 42.17, 12.82 41.86, 12.43 41.64, 12.43 41.65, 12.19 41.92))')
        layer.full_clean()
        layer.save()

        # creating node with same coordinates should not be an issue
        new_node = Node(**{
            'name': 'new_node',
            'slug': 'new_node',
            'layer': layer,
            'geometry': 'POINT (50.0 50.0)'
        })

        try:
            new_node.full_clean()
        except ValidationError as e:
            self.assertIn(_('Node must be inside layer area'), e.messages)
        else:
            self.fail('validation not working as expected')

        # if area is a point the contains check won't be done
        layer.area = GEOSGeometry('POINT (30.0 30.0)')
        layer.full_clean()
        layer.save()
        new_node.full_clean()

    def test_node_geometry_distance_and_area(self):
        """ test minimum distance check between nodes """
        self.client.login(username='admin', password='tester')

        url = reverse('api_node_list')

        json_data = {
            "layer": "rome",
            "name": "test_distance",
            "slug": "test_distance",
            "address": "via dei test",
            "description": "",
            "geometry": json.loads(GEOSGeometry("POINT (12.5822391919000012 41.8720419276999820)").json),
        }
        layer = Layer.objects.get(pk=1)
        layer.nodes_minimum_distance = 100
        layer.full_clean()
        layer.save()

        # Node coordinates don't respect minimum distance. Insert should fail because coords are near to already existing PoI ( fusolab )
        response = self.client.post(url, json.dumps(json_data), content_type='application/json')
        self.assertEqual(400, response.status_code)

        # Node coordinates respect minimum distance. Insert should succed
        json_data['geometry'] = json.loads(GEOSGeometry("POINT (12.7822391919 41.8720419277)").json)
        response = self.client.post(url, json.dumps(json_data), content_type='application/json')
        self.assertEqual(201, response.status_code)

        # Disable minimum distance control in layer and update node inserting coords too near. Insert should succed
        layer.nodes_minimum_distance = 0
        layer.save()
        json_data['geometry'] = json.loads(GEOSGeometry("POINT (12.5822391917 41.872042278)").json)
        n = Node.objects.get(slug='test_distance')
        node_slug = n.slug
        url = reverse('api_node_details', args=[node_slug])
        response = self.client.put(url, json.dumps(json_data), content_type='application/json')
        self.assertEqual(200, response.status_code)

        # re-enable minimum distance and update again with coords too near. Insert should fail
        layer.nodes_minimum_distance = 100
        layer.save()
        url = reverse('api_node_details', args=[node_slug])
        response = self.client.put(url, json.dumps(json_data), content_type='application/json')
        self.assertEqual(400, response.status_code)

        # Defining an area for the layer and testing if node is inside the area
        layer.area = GEOSGeometry('POLYGON ((12.19 41.92, 12.58 42.17, 12.82 41.86, 12.43 41.64, 12.43 41.65, 12.19 41.92))')
        layer.save()

        # Node update should fail because coords are outside layer area
        json_data['geometry'] = json.loads(GEOSGeometry("POINT (50 50)").json)
        url = reverse('api_node_details', args=[node_slug])
        response = self.client.put(url, json.dumps(json_data), content_type='application/json')
        self.assertEqual(400, response.status_code)

        # Node update should succeed because coords are inside layer area and respect minimum distance
        json_data['geometry'] = json.loads(GEOSGeometry("POINT (12.7822391919 41.8720419277)").json)
        url = reverse('api_node_details', args=[node_slug])
        response = self.client.put(url, json.dumps(json_data), content_type='application/json')
        self.assertEqual(200, response.status_code)

        # Node update should succeed because layer area is disabled
        layer.area = GEOSGeometry("POINT (12.7822391919 41.8720419277)")
        layer.save()
        json_data['geometry'] = json.loads(GEOSGeometry("POINT (50 50)").json)
        url = reverse('api_node_details', args=[node_slug])
        response = self.client.put(url, json.dumps(json_data), content_type='application/json')
        self.assertEqual(200, response.status_code)

        # re-enable minimum distance
        layer.nodes_minimum_distance = 100
        layer.save()

        # delete new nodes just added before
        n.delete()

    def test_layer_center(self):
        l = Layer.objects.first()
        self.assertIsInstance(l.center, Point)
        self.assertEqual(l.center, l.area)
        l.area = GEOSGeometry('POLYGON ((12.19 41.92, 12.58 42.17, 12.82 41.86, 12.43 41.64, 12.43 41.65, 12.19 41.92))')
        l.save()
        self.assertIsInstance(l.center, Point)
        self.assertNotEqual(l.center, l.area)
        l.area = None
        self.assertIsNone(l.center)

    def test_external_layer_nodes_geojson(self):
        """ test node geojson list """
        url = reverse('api_layer_nodes_geojson', args=['vienna'])
        # GET: 200
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)

    def test_external_layer_nodes(self):
        """ test node geojson list """
        url = reverse('api_layer_nodes_list', args=['vienna'])
        # GET: 200
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)

    def test_layer_nodes_geojson_pagination(self):
        url = reverse('api_layer_nodes_geojson', args=['rome'])
        # ensure all results returned by default
        response = self.client.get(url)
        count = Node.objects.filter(layer__slug='rome').published().access_level_up_to('public').count()
        self.assertEqual(len(response.data['features']), count)
        # ensure pagination doesn't break geojson format
        response = self.client.get(url, {'limit': '1'})
        self.assertIn('type', response.data)
        self.assertIn('features', response.data)
        self.assertEqual(len(response.data['features']), 1)



"""
Django database routers are described here:
https://docs.djangoproject.com/en/dev/topics/db/multi-db/#using-routers
"""


class DefaultRouter(object):
    def db_for_read(self, model, **hints):
        """
        Reads from nodeshot2 db
        """
        if model._meta.app_label != 'oldimporter':
            return 'default'
        return None

    def db_for_write(self, model, **hints):
        """
        Writes to nodeshot2 db
        """
        if model._meta.app_label != 'oldimporter':
            return 'default'
        return None

    def allow_relation(self, obj1, obj2, **hints):
        """
        Relations between objects are allowed between nodeshot2 objects only
        """
        if obj1._meta.app_label != 'oldimporter' and \
           obj2._meta.app_label != 'oldimporter':
           return True
        return None

    def allow_syncdb(self, db, model):
        """
        Make sure the old_nodeshot app only appears in the 'old_nodeshot' database
        """
        if db != 'old_nodeshot' and model._meta.app_label != 'oldimporter':
            return True
        return None
    
    allow_migrate = allow_syncdb


class OldNodeshotRouter(object):
    def db_for_read(self, model, **hints):
        """
        Reads old nodeshot models from old_nodeshot db.
        """
        if model._meta.app_label == 'oldimporter':
            return 'old_nodeshot'
        return None

    def db_for_write(self, model, **hints):
        """
        Writes not allowed
        """
        if model._meta.app_label == 'oldimporter':
            return 'old_nodeshot'
        return None

    def allow_relation(self, obj1, obj2, **hints):
        """
        Relations between objects are allowed between old_nodeshot objects only
        """
        if obj1._meta.app_label == 'oldimporter' and \
           obj2._meta.app_label == 'oldimporter':
           return True
        return None

    def allow_syncdb(self, db, model):
        """
        Make sure the old_nodeshot app only appears in the 'old_nodeshot' database
        """
        if db != 'old_nodeshot' or model._meta.app_label != 'oldimporter':
            return False
        return True
    
    allow_migrate = allow_syncdb



"""
Dependencies:
    * nodeshot.core.nodes
    * nodeshot.networking.net
"""

from nodeshot.core.base.utils import check_dependencies

check_dependencies(
    dependencies=[
        'nodeshot.core.nodes',
        'nodeshot.core.layers',
        'nodeshot.networking.net'
    ],
    module='nodeshot.networking.links'
)


from .link import Link


__all__ = ['Link']


# ------ Add relationship to NodeDetailSerializer ------ #

from nodeshot.core.nodes.serializers import ExtensibleNodeSerializer

ExtensibleNodeSerializer.add_relationship(**{
    'name': 'links',
    'view_name': 'api_node_links',
    'lookup_field': 'slug'
})



from django.db import models
from django.utils.translation import ugettext_lazy as _

from nodeshot.core.base.models import BaseAccessLevel
from nodeshot.core.base.managers import AccessLevelManager

from .choices import LOGIN_TYPES


class ServiceLogin(BaseAccessLevel):
    service = models.ForeignKey('services.Service',
                                verbose_name=_('service'))
    type = models.SmallIntegerField(_('type'), max_length=2,
                                    choices=LOGIN_TYPES,
                                    default=LOGIN_TYPES[0][0])
    username = models.CharField(_('username'), max_length=30)  # space should not be allowed
    password = models.CharField(_('password'), max_length=128)
    description = models.CharField(_('description'), max_length=255, blank=True, null=True)
    
    objects = AccessLevelManager()
    
    class Meta:
        app_label= 'services'
        db_table = 'service_logins'
        permissions = (('can_view_service_login', 'Can view service logins'),)
        verbose_name = _('login')
        verbose_name_plural = _('logins')
        unique_together = ('username', 'service')
    
    def __unicode__(self):
        return '%s - %s' % (self.username, self.service) 


from django.core.exceptions import ValidationError
from django.utils.translation import ugettext_lazy as _
from rest_framework.fields import WritableField
from netaddr import eui, ip


class MacAddressField(WritableField):
        """
        A field to handle mac address and avoid 500 internal server errors
        """
        
        def validate(self, value):
            """ ensure valid mac """
            super(MacAddressField, self).validate(value)
            
            try:
                eui.EUI(value)
            except eui.AddrFormatError:
                raise ValidationError(_('Invalid mac address'))
        
        def from_native(self, value):
            return value
    
        def to_native(self, value):
            return unicode(value)


class IPAddressField(WritableField):
        """
        A field to handle ip address and avoid 500 internal server errors
        """
        
        def validate(self, value):
            """ ensure valid ip address """
            super(IPAddressField, self).validate(value)
            
            try:
                ip.IPAddress(value)
            except (ip.AddrFormatError, ValueError):
                raise ValidationError(_('Invalid ip address'))
        
        def from_native(self, value):
            return value
    
        def to_native(self, value):
            return unicode(value)


class IPNetworkField(WritableField):
        """
        A field to handle ip network and avoid 500 internal server errors
        """
        
        def validate(self, value):
            """ ensure valid ip network """
            super(IPNetworkField, self).validate(value)
            
            try:
                ip.IPNetwork(value)
            except (ip.AddrFormatError, ValueError):
                raise ValidationError(_('Invalid ip network'))
        
        def from_native(self, value):
            return value
    
        def to_native(self, value):
            if value:
                return unicode(value)
            return value


from django import template
from leaflet.templatetags.leaflet_tags import leaflet_map


register = template.Library()


@register.inclusion_tag('leaflet/_load_map.html')
def nodeshot_map(name, callback=None, fitextent=True, creatediv=True, loadevent='load'):
    return leaflet_map(name, callback=None, fitextent=True, creatediv=True, loadevent='load')



from nodeshot.core.base.utils import check_dependencies
from layer import Layer


__all__ = ['Layer']


check_dependencies(
    dependencies='nodeshot.core.nodes',
    module='nodeshot.core.layers'
)


# ------ Add relationship to ExtensibleNodeSerializer ------ #

from nodeshot.core.nodes.serializers import ExtensibleNodeSerializer

ExtensibleNodeSerializer.add_relationship(**{
    'name': 'layer',
    'view_name': 'api_layer_detail',
    'lookup_field': 'layer.slug'
})



from django.db import models
from django.utils.translation import ugettext_lazy as _

from nodeshot.core.base.models import BaseDate
from choices import POLARIZATION_CHOICES

from . import ImageMixin, DeviceModel, Manufacturer


class AntennaModel(BaseDate, ImageMixin):
    """
    Antenna Type Model
    Eg: Airmax Sector, Rocket Dish, ecc.
    """
    name = models.CharField(_('name'), max_length=50, unique=True)
    device_model = models.OneToOneField(DeviceModel, blank=True, null=True, help_text=_('specify only if it\'s an integrated antenna'))
    manufacturer = models.ForeignKey(Manufacturer)
    gain = models.DecimalField(_('gain'), max_digits=4, decimal_places=1, help_text=_('dBi'))
    polarization = models.SmallIntegerField(_('Polarization'), choices=POLARIZATION_CHOICES, blank=True, null=True)
    freq_range_lower = models.IntegerField(_('minimum Frequency'), help_text=_('MHz'))
    freq_range_higher = models.IntegerField(_('maximum Frequency'), help_text=_('MHz'))
    beamwidth_h = models.DecimalField(_('hpol Beamwidth'), max_digits=4, decimal_places=1, help_text=_('degrees'))
    beamwidth_v = models.DecimalField(_('vpol Beamwidth'), max_digits=4, decimal_places=1, help_text=_('degrees'))
    image = models.ImageField(_('image'), upload_to='antennas/images/', blank=True)
    datasheet = models.FileField(_('datasheet'), upload_to='antennas/datasheets/', blank=True)
    
    def __unicode__(self):
        return self.name
    
    class Meta:
        app_label= 'hardware'
        db_table = 'hardware_antenna_model'



"""
nodeshot.community.mailing unit tests
"""

import datetime

from django.test import TestCase
from django.utils.translation import ugettext as _
from django.core import mail
from django.core.urlresolvers import reverse
from django.core.exceptions import ValidationError
from django.db.models import Q
from django.utils.timezone import utc
from django.contrib.contenttypes.models import ContentType
from django.contrib.auth import get_user_model
User = get_user_model()

from nodeshot.core.layers.models import Layer
from nodeshot.core.nodes.models import Node
from nodeshot.core.base.tests import user_fixtures
from nodeshot.community.profiles.settings import EMAIL_CONFIRMATION

from .models import Inward, Outward
from .models.choices import FILTERS

if EMAIL_CONFIRMATION:
    from nodeshot.community.profiles.models import EmailAddress


class MailingTest(TestCase):
    fixtures = [
        'initial_data.json',
        user_fixtures,
        'test_layers.json',
        'test_status.json',
        'test_nodes.json'
    ]

    def setUp(self):
        # create outward record
        self.message = Outward.objects.create(
            status=0,
            subject='test message',
            message='This is a test message, be sure this text is longer than the minimum required',
            is_scheduled=False,
            is_filtered=False
        )
        self.msg = Outward(
            status=0,
            subject='This is a test',
            message=self.message.message,
        )
        mail.outbox = []
        if EMAIL_CONFIRMATION:
            # set all email addresses as verified
            for email_address in EmailAddress.objects.all():
                email_address.verified = True
                email_address.save()

    def test_inward_model(self):
        """ ensure inward model validation works as expected """
        message = Inward(message='this is a test to ensure inward model validation works as expected')
        content_type = ContentType.objects.only('id', 'model').get(model='node')
        message.content_type = content_type
        message.object_id = 1
        user = User.objects.get(pk=1)
        # ensure validation error
        try:
            message.full_clean()
            self.fail('ValidationError not raised')
        except ValidationError as e:
            self.assertIn(_('If sender is not specified from_name and from_email must be filled in'), e.message_dict['__all__'][0])
        # ensure validation error
        try:
            message.from_email = user.email
            message.from_name = user.get_full_name()
            message.full_clean()
            self.fail('ValidationError not raised')
        except ValidationError as e:
            self.assertIn(_('This field cannot be blank'), e.message_dict['user'][0])
        # ensure no validation error
        message.full_clean(exclude=['user'])
        # empty from_name and from_email, we need to ensure they'll filled automatically
        message.from_name = None
        message.from_email = None
        # set user
        message.user = user
        message.full_clean()
        message.save()
        # expect correct mail sent
        self.assertEqual(len(mail.outbox), 1)
        self.assertEqual(message.status, 1)
        self.assertEqual(message.from_name, user.get_full_name())
        self.assertEqual(message.from_email, user.email)

    def test_contact_node_api(self):
        """ ensure contact node api works as expected """
        url = reverse('api_node_contact', args=['fusolab'])
        self.client.login(username='admin', password='tester')
        mail.outbox = []

        response = self.client.get(url)
        self.assertEqual(response.status_code, 405)

        response = self.client.post(url, {'message': 'ensure contact node api works as expected'})
        self.assertEqual(response.status_code, 201)
        self.assertEqual(len(mail.outbox), 1)
        self.assertEqual(Inward.objects.count(), 1)

    def test_contact_user_api(self):
        """ ensure contact user api works as expected """
        url = reverse('api_user_contact', args=['romano'])
        self.client.login(username='admin', password='tester')
        mail.outbox = []

        response = self.client.get(url)
        self.assertEqual(response.status_code, 405)

        response = self.client.post(url, {'message': 'ensure contact user api works as expected'})
        self.assertEqual(response.status_code, 201)
        self.assertEqual(len(mail.outbox), 1)
        self.assertEqual(Inward.objects.count(), 1)

    def test_contact_layer_api(self):
        """ ensure contact layer api works as expected """
        url = reverse('api_layer_contact', args=['rome'])
        self.client.login(username='admin', password='tester')
        mail.outbox = []

        response = self.client.get(url)
        self.assertEqual(response.status_code, 405)

        response = self.client.post(url, {'message': 'ensure contact user api works as expected'})
        self.assertEqual(response.status_code, 201)
        self.assertEqual(len(mail.outbox), 1)
        self.assertEqual(Inward.objects.count(), 1)

        vienna = Layer.objects.get(slug='vienna')
        vienna.email = ''
        vienna.save()

        # cannot be contacted because no email nor mantainers specified
        url = reverse('api_layer_contact', args=['vienna'])
        response = self.client.get(url)
        self.assertEqual(response.status_code, 400)

        response = self.client.post(url, {'message': 'ensure contact user api works as expected'})
        self.assertEqual(response.status_code, 400)

    def test_contact_api_validation_error(self):
        url = reverse('api_layer_contact', args=['rome'])
        self.client.login(username='admin', password='tester')
        mail.outbox = []

        response = self.client.post(url, {'message': 'short'})
        self.assertEqual(len(mail.outbox), 0)
        self.assertEqual(Inward.objects.count(), 0)
        self.assertEqual(response.status_code, 400)
        self.assertIn('Ensure this value has at least', response.data['message'][0])

    def test_no_filter(self):
        """ *** Test no filtering, send to all *** """
        # count users
        users = User.objects.filter(is_active=True)
        recipients = self.message.get_recipients()

        # simplest case: send to all
        # fail if recipient list length and user list length differ
        self.assertEqual(len(recipients), len(users))
        # fail if user emails are not in recipients
        for user in users:
            self.assertTrue(user.email in recipients)

    def test_outward_admin_send_action(self):
        """ simulate sending mails from the admin """
        from django.http.request import HttpRequest
        from django.contrib.admin.sites import AdminSite
        from .admin import OutwardAdmin
        admin = OutwardAdmin(Outward, AdminSite())
        request = HttpRequest()
        self.assertEqual(Outward.objects.count(), 1)
        # trigger send action
        admin.send(request, Outward.objects.all())
        # ensure outward message has status == sent
        message = Outward.objects.first()
        self.assertEqual(message.status, 2)
        # ensure messages have been sent to users
        self.assertEqual(len(mail.outbox), User.objects.count())

    def test_group_filtering(self):
        """ *** Test group filtering in multiple combinations *** """
        combinations = [
            '1', '2', '3', '0',
            '1,2', '1,3', '1,0', '2,3', '2,0', '3,0',
            '1,2,3', '1,2,0', '1,3,0', '2,3,0',
            '1,2,3,0'
        ]
        # prepare record
        message = self.message
        message.is_filtered = True
        message.filters = '1'

        for combo in combinations:
            # test multiple groups
            groups = combo.split(',')
            # init empty Q
            q = Q()
            for group in groups:
                # if not superuser case
                if group != '0':
                    # add another group to the Q
                    q = q | Q(groups=group)
                # superuser case
                else:
                    # group 0 wouldn't be correct, therefore we use is_superuser=True
                    q = q | Q(is_superuser=True)
            # and is_active = True
            q = q & Q(is_active=True)
            # retrieve users
            users = User.objects.filter(q)
            message.groups = combo
            recipients = message.get_recipients()
            # fail if recipient list length and user list length differ
            self.assertEqual(len(recipients), len(users))
            # fail if user emails are not in recipients
            for user in users:
                self.assertTrue(user.email in recipients)

    def test_layer_filtering(self):
        """ *** Test layer filtering in multiple combinations *** """
        combinations = [
            '1', '2', '3',
            '1,2', '1,3', '2,3',
            '1,2,3'
        ]
        # prepare record
        message = self.message
        message.is_filtered = True
        message.filters = '2'

        for combo in combinations:
            layers = combo.split(',')
            # init empty Q
            q = Q()
            for layer in layers:
                q = q | Q(layer=layer)
            # retrieve nodes
            nodes = Node.objects.filter(q)
            # layers id list
            message.layers = [int(layer) for layer in layers]
            # retrieve recipients according to model code
            recipients = message.get_recipients()
            # retrieve list of emails
            emails = []
            for node in nodes:
                if node.user.email not in emails:
                    emails += [node.user.email]
            # fail if recipient list length and user list length differ
            self.assertEqual(len(recipients), len(emails))
            # fail if user emails are not in recipients
            for email in emails:
                self.assertTrue(email in recipients)

    def test_user_filtering(self):
        """ *** Test recipient filtering by user *** """
        combinations = [
            '1', '2', '3', '4', '5', '6', '7', '8',
            '1,2', '2,3', '3,4', '5,6'
            '1,2,3', '3,4,5', '6,7,8'
            '1,2,3,4', '5,6,7,8',
            '1,2,3,4,5,6,7,8'
        ]
        # prepare record
        message = self.message
        message.is_filtered = True
        message.filters = '3'

        for combo in combinations:
            # users id list
            users_string = combo.split(',')
            user_ids = [int(user) for user in users_string]
            # selected users
            message.users = user_ids
            # retrieve recipients according to the model code
            recipients = message.get_recipients()
            # init empty Q
            q = Q()
            # retrieve users
            for user in user_ids:
                q = q | Q(id=user)
            # only active users
            q = q & Q(is_active=True)
            users = User.objects.filter(q)
            # fail if recipient list length and user list length differ
            self.assertEqual(len(recipients), len(users))
            # fail if user emails are not in recipients
            for user in users:
                self.assertTrue(user.email in recipients)

    def test_group_and_layer_filtering(self):
        """ *** Test group & layer filtering in multiple combinations *** """
        combinations = [
            {'groups': '1', 'layers': '1'},
            {'groups': '2', 'layers': '2'},
            {'groups': '1,2', 'layers': '1'},
            {'groups': '1,2,3', 'layers': '1,2'},
            {'groups': '1,2', 'layers': '1,2,3'},
            {'groups': '3', 'layers': '1,2,3'}
        ]
        # prepare record
        message = self.message
        message.is_filtered = True
        message.filters = '1,2'

        for combo in combinations:
            groups = combo['groups'].split(',')
            layers = combo['layers'].split(',')

            # GROUPS
            q1 = Q()
            for group in groups:
                # if not superuser case
                if group != '0':
                    # add another group to the Q
                    q1 = q1 | Q(user__groups=group)
                # superuser case
                else:
                    # group 0 wouldn't be correct, therefore we use is_superuser=True
                    q1 = q1 | Q(user__is_superuser=True)
            # and is_active = True
            q1 = q1 & Q(user__is_active=True)

            # LAYERS
            q2 = Q()
            for layer in layers:
                q2 = q2 | Q(layer=layer)
            # retrieve nodes
            nodes = Node.objects.filter(q1 & q2).select_related()

            # message group & layers
            message.groups = combo['groups']
            message.layers = [int(layer) for layer in layers]

            # retrieve recipients according to model code
            recipients = message.get_recipients()
            # retrieve list of emails
            emails = []
            for node in nodes:
                if node.user.email not in emails:
                    emails += [node.user.email]
            # fail if recipient list length and user list length differ
            self.assertEqual(len(recipients), len(emails))
            # fail if user emails are not in recipients
            for email in emails:
                self.assertTrue(email in recipients)

    def test_user_and_layer_filtering(self):
        """ *** Test recipient filtering by user & layers *** """
        combinations = [
            {'users': '1', 'layers': '1'},
            {'users': '2', 'layers': '2'},
            {'users': '1,2', 'layers': '1'},
            {'users': '1,2,3', 'layers': '1,2'},
            {'users': '1,2', 'layers': '1,2,3'},
            {'users': '3', 'layers': '1,2,3'},
            {'users': '3,4,5,6,7', 'layers': '1'}
        ]
        # prepare record
        message = self.message
        message.is_filtered = True
        message.filters = '2,3'

        for combo in combinations:
            users = combo['users'].split(',')
            layers = combo['layers'].split(',')

            # ZONES
            q1 = Q()
            for layer in layers:
                q1 = q1 | Q(layer=layer)
            # retrieve nodes
            nodes = Node.objects.filter(q1).select_related()

            # prepare Q object for user query
            q2 = Q()
            for user in users:
                q2 = q2 | Q(pk=user)
            q2 = q2 & Q(is_active=True)

            # message users & layers
            message.users = [int(user) for user in users]
            message.layers = [int(layer) for layer in layers]

            # retrieve chosen users
            users = User.objects.filter(q2)

            # retrieve recipients according to model code
            recipients = message.get_recipients()

            # retrieve list of emails
            emails = []
            for node in nodes:
                if node.user.email not in emails:
                    emails.append(node.user.email)

            # add emails of selected users if necessary
            for user in users:
                if user.email not in emails:
                    emails.append(user.email)

            # fail if recipient list length and user list length differ
            self.assertEqual(len(recipients), len(emails))
            # fail if user emails are not in recipients
            for email in emails:
                self.assertTrue(email in recipients)

    # TODO:
    # test html/plain-text format

    def test_outward_validation_scheduled1(self):
        """ *** A scheduled message without any value for scheduled date and time should not pass validation *** """
        self.msg.is_scheduled = 1
        self.msg.scheduled_date = None
        self.msg.scheduled_time = None
        self.assertRaises(ValidationError, self.msg.full_clean)

    def test_outward_validation_scheduled2(self):
        """ *** A scheduled message without any value for scheduled time should not pass validation *** """
        self.msg.is_scheduled = 1
        self.msg.scheduled_time = None
        self.msg.scheduled_date = datetime.datetime.utcnow().replace(tzinfo=utc).date() + datetime.timedelta(days=1)
        self.assertRaises(ValidationError, self.msg.full_clean)

    def test_outward_validation_scheduled3(self):
        """ *** A scheduled message without any value for scheduled date should not pass validation *** """
        self.msg.is_scheduled = 1
        self.msg.scheduled_date = None
        self.msg.scheduled_time = '00'
        self.assertRaises(ValidationError, self.msg.full_clean)

    def test_outward_validation_scheduled4(self):
        """ *** A scheduled message with both date and time should pass validation *** """
        self.msg.is_scheduled = 1
        self.msg.scheduled_date = datetime.datetime.utcnow().replace(tzinfo=utc).date() + datetime.timedelta(days=1)
        self.msg.scheduled_time = '00'
        self.msg.full_clean()

    def test_outward_validate_scheduled5(self):
        """ *** A new message with a past scheduled date should not pass validation *** """
        self.msg.is_scheduled = 1
        self.msg.scheduled_date = datetime.datetime.utcnow().replace(tzinfo=utc).date() - datetime.timedelta(days=1)
        self.msg.scheduled_time = '00'
        self.assertRaises(ValidationError, self.msg.full_clean)

    def test_outward_validate_filters1(self):
        """ *** A new message with is_filtered == True and no selected filter should not pass validation *** """
        self.msg.is_scheduled = 0
        self.msg.scheduled_date = None
        self.msg.scheduled_time = None
        self.msg.is_filtered = 1
        self.assertRaises(ValidationError, self.msg.full_clean)

    def test_outward_validate_filters2(self):
        """ *** A new message with group filtering active but not selected group should not pass validation *** """
        self.msg.is_scheduled = 0
        self.msg.is_filtered = 1
        self.msg.filters = '%s' % FILTERS.get('groups')
        self.msg.groups = ''
        self.assertRaises(ValidationError, self.msg.full_clean)

    # the following two validation routine require more cumbersome procedures ... can't be bothered right now!

    # def test_outward_validate_filters3(self):
    #    """ *** A new message with user filtering active but not selected group should not pass validation *** """
    #    self.msg.is_scheduled = 0
    #    self.msg.is_filtered = 1
    #    self.msg.filters = '%s' % FILTERS.get('users')
    #    self.msg.users = []
    #    self.assertRaises(ValidationError, self.msg.full_clean)
    #
    # def test_outward_validate_filters4(self):
    #    """ *** A new message with layer filtering active but not selected group should not pass validation *** """
    #    self.msg.is_scheduled = 0
    #    self.msg.is_filtered = 1
    #    self.msg.filters = '%s' % FILTERS.get('layers')
    #    self.msg.layers = []
    #    self.assertRaises(ValidationError, self.msg.full_clean)
    #
    # def test_outward_plaintext(self):
    #    """ *** TODO: write a test that verifies email is correctly sent as plain text *** """
    #    self.assertEqual(False, True)
    #
    # def test_outward_html(self):
    #    """ *** TODO: write a test that verifies email is correctly sent both as plain text and HTML *** """
    #    self.assertEqual(False, True)



import time
import simplejson as json
from threading import Thread

import tornado.web
import tornado.ioloop

from .handlers import WebSocketHandler
from . import ADDRESS, PORT, PUBLIC_PIPE, PRIVATE_PIPE  # contained in __init__.py


application = tornado.web.Application([
    (r'/', WebSocketHandler),
])


def public_broadcaster():
    """
    Thread which runs in parallel and constantly checks for new messages
    in the public pipe and broadcasts them publicly to all connected clients.
    """
    while __websocket_server_running__:
        pipein = open(PUBLIC_PIPE, 'r')
        line = pipein.readline().replace('\n', '').replace('\r', '')
        if line != '':
            WebSocketHandler.broadcast(line)
            print line
            
            remaining_lines = pipein.read()
            pipein.close()
            pipeout = open(PUBLIC_PIPE, 'w')
            pipeout.write(remaining_lines)
            pipeout.close()
        else:
            pipein.close()
        
        time.sleep(0.05)

public_broadcaster_thread = Thread(target=public_broadcaster, args=[])
public_broadcaster_thread.deamon = True


def private_messenger():
    """
    Thread which runs in parallel and constantly checks for new messages
    in the private pipe and sends them to the specific client.
    If client is not connected the message is discarded.
    """
    while __websocket_server_running__:
        pipein = open(PRIVATE_PIPE, 'r')
        line = pipein.readline().replace('\n', '').replace('\r', '')
        if line != '':
            message = json.loads(line)
            WebSocketHandler.send_private_message(user_id=message['user_id'],
                                                  message=message)
            print line
            
            remaining_lines = pipein.read()
            pipein.close()
            pipeout = open(PRIVATE_PIPE, 'w')
            pipeout.write(remaining_lines)
            pipeout.close()
        else:
            pipein.close()
        
        time.sleep(0.05)

private_messenger_thread = Thread(target=private_messenger, args=[])
private_messenger_thread.deamon = True


def start():    
    global __websocket_server_running__
    __websocket_server_running__ = True
    
    application.listen(PORT, address=ADDRESS)
    websocktserver = tornado.ioloop.IOLoop.instance()
    
    try:
        print "\nStarted Tornado Wesocket Server at ws://%s:%s\n" % (ADDRESS, PORT)
        
        public_broadcaster_thread.start()
        private_messenger_thread.start()
        websocktserver.start()
    # on exit
    except (KeyboardInterrupt, SystemExit):
        __websocket_server_running__ = False
        websocktserver.stop()
        
        print "\nStopped Tornado Wesocket Server\n"



from django.db import models
from django.utils.translation import ugettext_lazy as _

from nodeshot.networking.net.models import Device
from . import DeviceModel, Antenna, AntennaModel


class DeviceToModelRel(models.Model):
    """ OneToOne relationship between net.Device and hardware.DeviceModel """
    device = models.OneToOneField(Device, verbose_name=_('device'), related_name='hardware')
    model = models.ForeignKey(DeviceModel)
    cpu = models.CharField(_('CPU'), max_length=255, blank=True)
    ram = models.IntegerField(_('RAM'), blank=True, help_text=_('bytes'))
    
    class Meta:
        app_label= 'hardware'
        db_table = 'hardware_device_to_model'
        verbose_name = _('Device Model Information')
        verbose_name_plural = _('Device Model Information')
    
    def __unicode__(self):
        return '%s (%s)' % (self.device.name, self.model.name)
    
    def save(self, *args, **kwargs):
        """ when creating a new record fill CPU and RAM info if available """
        adding_new = False
        if not self.pk or (not self.cpu and not self.ram):
            # if self.model.cpu is not empty
            if self.model.cpu:
                self.cpu = self.model.cpu
            # if self.model.ram is not empty
            if self.model.ram:
                self.ram = self.model.ram
            # mark to add a new antenna
            adding_new = True
        # perform save
        super(DeviceToModelRel, self).save(*args, **kwargs)
        # after Device2Model object has been saved
        try:
            # does the device model have an integrated antenna?
            antenna_model = self.model.antennamodel
        except AntennaModel.DoesNotExist:
            # if not antenna_model is False
            antenna_model = False
        # if we are adding a new device2model and the device model has an integrated antenna
        if adding_new and antenna_model:
            # create new Antenna object
            antenna = Antenna(
                device=self.device,
                model=self.model.antennamodel
            )
            # retrieve wireless interfaces and assign it to the antenna object if possible
            wireless_interfaces = self.device.interface_set.filter(type=2)
            if len(wireless_interfaces) > 0:
                antenna.radio = wireless_interfaces[0]
            # save antenna
            antenna.save()



from django.db import models
from django.utils.translation import ugettext_lazy as _

from netfields import InetAddressField

from nodeshot.networking.net.models import Interface
from nodeshot.networking.net.models.choices import INTERFACE_TYPES

from ...managers import NetAccessLevelManager


class Tunnel(Interface):
    """ Tunnel Interface """
    sap = models.CharField(max_length=10, null=True, blank=True)
    protocol = models.CharField(max_length=10, help_text=_('eg: GRE'), blank=True, null=True)
    endpoint = InetAddressField(verbose_name=_('end point'), blank=True, null=True)
    
    objects = NetAccessLevelManager()
    
    class Meta:
        app_label = 'net'
        db_table = 'net_interface_tunnel'
        verbose_name = _('tunnel interface')
        verbose_name_plural = _('tunnel interfaces')
    
    def save(self, *args, **kwargs):
        """ automatically set Interface.type to tunnel """
        self.type = INTERFACE_TYPES.get('tunnel')
        super(Tunnel, self).save(*args, **kwargs)


from django.db import models
from django.utils.translation import ugettext_lazy as _
from nodeshot.core.base.models import BaseDate
from choices import ROUTING_PROTOCOLS


class RoutingProtocol(BaseDate):
    """ Routing Protocol Model """
    name = models.CharField(_('name'), max_length=50, choices=ROUTING_PROTOCOLS)
    version = models.CharField(_('version'), max_length=128, blank=True)
    
    class Meta:
        app_label = 'net'
        db_table = 'net_routing_protocol'
        unique_together = ('name', 'version')
    
    def __unicode__(self):
        return '%s %s' % (self.name, self.version)


from django.db.models import Manager
from django.contrib.gis.db.models import GeoManager
from django.db.models.query import QuerySet
from django.contrib.gis.db.models.query import GeoQuerySet

from django_hstore.query import HStoreQuerySet, HStoreGeoQuerySet
from django_hstore.managers import HStoreManager, HStoreGeoManager

from nodeshot.core.base.choices import ACCESS_LEVELS


# -------- MIXINS -------- #


class BaseUtilityMixin(object):
    """ add some goodies for development """
    
    def slice(self, order_by='pk', n=None):
        """ return n objects according to specified ordering """
        if n is not None and n < 0:
            raise ValueError('slice parameter cannot be negative')
        
        queryset = self.order_by(order_by)
        
        if n is None:
            return queryset[0]
        else:
            return queryset[0:n]
    
    def find(self, pk):
        return self.get(pk=pk)


class PublishedMixin(BaseUtilityMixin):
    """ adds published filter to queryset """
    
    def published(self):
        """ return only published items """
        return self.filter(is_published=True)


class ACLMixin(BaseUtilityMixin):
    """ adds acl filters to queryset """
    
    def access_level_up_to(self, access_level):
        """ returns all items that have an access level equal or lower than the one specified """
        # if access_level is number
        if isinstance(access_level, int):
            value = access_level
        # else if is string get the numeric value
        else:
            value = ACCESS_LEVELS.get(access_level)
        # return queryset
        return self.filter(access_level__lte=value)
    
    def accessible_to(self, user):
        """
        returns all the items that are accessible to the specified user
        if user is not authenticated will return public items
        
        :param user: an user instance
        """
        if user.is_superuser:
            try:
                queryset = self.get_query_set()
            except AttributeError:
                queryset = self
        elif user.is_authenticated():
            # get user group (higher id)
            group = user.groups.all().order_by('-id')[0]
            queryset = self.filter(access_level__lte=ACCESS_LEVELS.get(group.name))
        else:
            queryset = self.filter(access_level__lte=ACCESS_LEVELS.get('public'))
        return queryset


class ExtendedManagerMixin(BaseUtilityMixin):
    """ add this mixin to add  support for chainable custom methods to your manager """
    
    def __getattr__(self, attr, *args):
        try:
            return getattr(self.__class__, attr, *args)
        except AttributeError:
            return getattr(self.get_query_set(), attr, *args)


# -------- QUERYSETS -------- #


class PublishedQuerySet(QuerySet, PublishedMixin):
    """ Custom queryset to filter only published items """
    pass


class GeoPublishedQuerySet(GeoQuerySet, PublishedMixin):
    """ PublishedQuerySet with GeoDjango queryset """
    pass


class AccessLevelQuerySet(QuerySet, ACLMixin):
    """ Custom queryset to filter depending on access levels """
    pass


class GeoAccessLevelQuerySet(GeoQuerySet, ACLMixin):
    """ AccessLevelQuerySet with GeoDjango queryset """
    pass


class AccessLevelPublishedQuerySet(QuerySet, ACLMixin, PublishedMixin):
    """ AccessLevelQuerySet and PublishedQuerySet """
    pass


class GeoAccessLevelPublishedQuerySet(GeoQuerySet, ACLMixin, PublishedMixin):
    """ AccessLevelQuerySet, PublishedQuerySet with GeoDjango queryset """
    pass


class HStoreAccessLevelQuerySet(HStoreQuerySet, ACLMixin):
    """ HStoreQuerySet, ACLMixin queryset """
    pass


class HStoreGeoPublishedQuerySet(HStoreGeoQuerySet, PublishedMixin):
    """ HStoreGeoQuerySet and PublishedMixin """
    pass


class HStoreGeoAccessLevelQuerySet(HStoreGeoQuerySet, ACLMixin):
    """ HStoreGeoQuerySet and AccessLevel """
    pass


class HStoreGeoAccessLevelPublishedQuerySet(HStoreGeoQuerySet, ACLMixin, PublishedMixin):
    """ HStoreGeoQuerySet, AccessLevelQuerySet, PublishedQuerySet with GeoDjango queryset """
    pass



# -------- MANAGERS -------- #


class NodeshotDefaultManager(Manager, ExtendedManagerMixin):
    """ Simple Manager that implements the BaseUtilityMixin methods """
    pass


class PublishedManager(Manager, ExtendedManagerMixin, PublishedMixin):
    """ Returns published items """
    
    def get_query_set(self): 
        return PublishedQuerySet(self.model, using=self._db)


class GeoPublishedManager(GeoManager, ExtendedManagerMixin, PublishedMixin):
    """ PublishedManager and GeoManager in one """
    
    def get_query_set(self): 
        return GeoPublishedQuerySet(self.model, using=self._db)


class GeoAccessLevelManager(GeoManager, ExtendedManagerMixin, ACLMixin):
    """ AccessLevelManager + Geodjango manager """
    
    def get_query_set(self): 
        return GeoAccessLevelQuerySet(self.model, using=self._db)


class AccessLevelManager(Manager, ExtendedManagerMixin, ACLMixin):
    """ Manager to filter depending on access level """

    def get_query_set(self): 
        return AccessLevelQuerySet(self.model, using=self._db)


class AccessLevelPublishedManager(Manager, ExtendedManagerMixin, ACLMixin, PublishedMixin):
    """
    AccessLeveManager and Publishedmanager in one
    """
    
    def get_query_set(self): 
        return AccessLevelPublishedQuerySet(self.model, using=self._db)


class GeoAccessLevelPublishedManager(GeoManager, ExtendedManagerMixin, ACLMixin, PublishedMixin):
    """
    GeoManager, AccessLeveManager and Publishedmanager in one
    """
    
    def get_query_set(self): 
        return GeoAccessLevelPublishedQuerySet(self.model, using=self._db)


class HStoreNodeshotManager(HStoreManager, ExtendedManagerMixin):
    """ HStoreManager + ExtendedManagerMixin """
    pass


class HStoreAccessLevelManager(HStoreManager, ExtendedManagerMixin, ACLMixin):
    """
    HStoreManager and AccessLeveManager in one
    """
    
    def get_query_set(self): 
        return HStoreAccessLevelQuerySet(self.model, using=self._db)


class HStoreGeoPublishedManager(HStoreGeoManager, ExtendedManagerMixin, PublishedMixin):
    """
    HStoreGeoManager and PublishedMixin in one
    """
    
    def get_query_set(self): 
        return HStoreGeoPublishedQuerySet(self.model, using=self._db)


class HStoreGeoAccessLevelManager(HStoreGeoManager, ExtendedManagerMixin, ACLMixin):
    """
    HStoreGeoManager and AccessLeveManager in one
    """
    
    def get_query_set(self): 
        return HStoreGeoAccessLevelQuerySet(self.model, using=self._db)


class HStoreGeoAccessLevelPublishedManager(HStoreGeoManager, ExtendedManagerMixin, ACLMixin, PublishedMixin):
    """
    HStoreManager, GeoManager, AccessLeveManager and Publishedmanager in one
    """
    
    def get_query_set(self): 
        return HStoreGeoAccessLevelPublishedQuerySet(self.model, using=self._db)



from django.db import models
from django.utils.translation import ugettext_lazy as _

from nodeshot.core.base.models import BaseOrdered
from nodeshot.core.base.fields import RGBColorField


class Status(BaseOrdered):
    """
    Status of a node, eg: active, potential, approved
    """
    name = models.CharField(_('name'), max_length=255,
                            help_text=_('label for this status, eg: active, approved, proposed'))
    slug = models.SlugField(max_length=75, db_index=True, unique=True)
    description = models.CharField(_('description'), max_length=255,
                                   help_text=_('this description will be used in the legend'))

    is_default = models.BooleanField(
        default=False,
        verbose_name=_('is default status?'),
        help_text=_('indicates whether this is the default status for new nodes;\
                    to change the default status to a new one just check and save,\
                    any other default will be automatically unchecked')
    )

    # map look and feel
    stroke_width = models.SmallIntegerField(
        blank=False,
        default=0,
        help_text=_('stroke of circles shown on map, set to 0 to disable')
    )
    fill_color = RGBColorField(_('fill colour'), blank=True)
    stroke_color = RGBColorField(_('stroke colour'), blank=True, default='#000000')
    text_color = RGBColorField(_('text colour'), blank=True, default='#FFFFFF')

    # needed to intercept changes to is_default
    _current_is_default = False

    class Meta:
        db_table = 'nodes_status'
        app_label = 'nodes'
        verbose_name = _('status')
        verbose_name_plural = _('status')
        ordering = ['order']

    def __unicode__(self):
        return self.name

    def __init__(self, *args, **kwargs):
        """ Fill _current_is_default """
        super(Status, self).__init__(*args, **kwargs)
        # set current is_default, but only if it is an existing status
        if self.pk:
            self._current_is_default = self.is_default

    def save(self, *args, **kwargs):
        """ intercepts changes to is_default """
        ignore_default_check = kwargs.pop('ignore_default_check', False)

        # if making this status the default one
        if self.is_default != self._current_is_default and self.is_default is True:
            # uncheck other default statuses first
            for status in self.__class__.objects.filter(is_default=True):
                status.is_default = False
                status.save(ignore_default_check=True)

        super(Status, self).save(*args, **kwargs)

        # in case there are no default statuses, make this one as the default one
        if self.__class__.objects.filter(is_default=True).count() == 0 and not ignore_default_check:
            self.is_default = True
            self.save()

        # update __current_status
        self._current_is_default = self.is_default



from django.db import models
from django.db.models import Avg
from django.utils.translation import ugettext_lazy as _
from django.conf import settings
from django.core.exceptions import ValidationError

from nodeshot.core.base.models import BaseDate
from nodeshot.core.nodes.models import Node
from nodeshot.core.layers.models import Layer

from .base import UpdateCountsMixin


class Rating(UpdateCountsMixin, BaseDate):
    """
    Rating model
    """
    # rating choices from 1 to 10
    RATING_CHOICES = [(n, '%d' % n) for n in range(1, 11)]

    node = models.ForeignKey(Node)
    user = models.ForeignKey(settings.AUTH_USER_MODEL)
    value = models.IntegerField(_('rating value'), choices=RATING_CHOICES)

    class Meta:
        app_label = 'participation'

    def __unicode__(self):
        if self.pk:
            return _('rating #%d for node %s') % (self.pk, self.node.name)
        else:
            return _('rating')

    def save(self, *args, **kwargs):
        """
        ensure users cannot rate the same node multiple times
        but let users change their rating
        """
        if not self.pk:
            old_ratings = Rating.objects.filter(user=self.user, node=self.node)
            for old_rating in old_ratings:
                old_rating.delete()
        super(Rating, self).save(*args, **kwargs)

    def update_count(self):
        """ updates rating count and rating average """
        node_rating_count = self.node.rating_count
        node_rating_count.rating_count = self.node.rating_set.count()
        node_rating_count.rating_avg = self.node.rating_set.aggregate(rate=Avg('value'))['rate']

        # if all ratings are deleted the value will be None!
        if node_rating_count.rating_avg is None:
            # set to 0 otherwise we'll get an exception
            node_rating_count.rating_avg = 0

        node_rating_count.save()

    def clean(self, *args, **kwargs):
        """
        Check if rating can be inserted for parent node or parent layer
        """
        if not self.pk:
            node = self.node
            layer = Layer.objects.get(pk=node.layer_id)
            if layer.participation_settings.rating_allowed is not True:
                raise ValidationError("Rating not allowed for this layer")
            if node.participation_settings.rating_allowed is not True:
                raise ValidationError("Rating not allowed for this node")



from django.db import models
from django.utils.translation import gettext_lazy as _

from django.utils.http import int_to_base36
from django.template.loader import render_to_string
from django.core.mail import send_mail
from django.contrib.auth.tokens import default_token_generator as token_generator

from nodeshot.core.base.utils import now

from ..settings import settings


class PasswordResetManager(models.Manager):
    """ Password Reset Manager """

    def create_for_user(self, user):
        """ create password reset for specified user """
        # support passing email address too
        if type(user) is unicode:
            from .profile import Profile as User
            user = User.objects.get(email=user)

        temp_key = token_generator.make_token(user)

        # save it to the password reset model
        password_reset = PasswordReset(user=user, temp_key=temp_key)
        password_reset.save()

        # send the password reset email
        subject = _("Password reset email sent")
        message = render_to_string("profiles/email_messages/password_reset_key_message.txt", {
            "user": user,
            "uid": int_to_base36(user.id),
            "temp_key": temp_key,
            "site_url": settings.SITE_URL,
            "site_name": settings.SITE_NAME
        })
        send_mail(subject, message, settings.DEFAULT_FROM_EMAIL, [user.email])

        return password_reset


class PasswordReset(models.Model):
    """
    Password reset Key
    """
    user = models.ForeignKey(settings.AUTH_USER_MODEL, verbose_name=_("user"))

    temp_key = models.CharField(_("temp_key"), max_length=100)
    timestamp = models.DateTimeField(_("timestamp"), default=now)
    reset = models.BooleanField(_("reset yet?"), default=False)

    objects = PasswordResetManager()

    class Meta:
        verbose_name = _('password reset')
        verbose_name_plural = _('password resets')
        app_label = 'profiles'

    def __unicode__(self):
        return "%s (key=%s, reset=%r)" % (
            self.user.username,
            self.temp_key,
            self.reset
        )



from django.http import Http404
from django.utils.translation import ugettext_lazy as _

from rest_framework import generics, permissions, authentication
from rest_framework.response import Response

from nodeshot.core.base.utils import Hider
from nodeshot.core.nodes.views import NodeList
from nodeshot.core.nodes.serializers import NodeGeoSerializer, PaginatedGeojsonNodeListSerializer

from .settings import REVERSION_ENABLED
from .models import Layer
from .serializers import *  # noqa


if REVERSION_ENABLED:
    from nodeshot.core.base.mixins import RevisionCreate, RevisionUpdate

    class LayerListBase(RevisionCreate, generics.ListCreateAPIView):
        pass

    class LayerDetailBase(RevisionUpdate, generics.RetrieveUpdateAPIView):
        pass
else:
    class LayerListBase(generics.ListCreateAPIView):
        pass

    class LayerDetailBase(generics.RetrieveUpdateAPIView):
        pass


class LayerList(LayerListBase):
    """
    Retrieve list of all layers.

    ### POST

    Create new layer if authorized (admins and allowed users only).
    """
    queryset = Layer.objects.published()
    permission_classes = (permissions.DjangoModelPermissionsOrAnonReadOnly, )
    authentication_classes = (authentication.SessionAuthentication,)
    serializer_class = LayerListSerializer
    pagination_serializer_class = PaginatedLayerListSerializer
    paginate_by_param = 'limit'
    paginate_by = None

layer_list = LayerList.as_view()


class LayerDetail(LayerDetailBase):
    """
    Retrieve details of specified layer.

    ### PUT & PATCH

    Edit specified layer
    """
    permission_classes = (permissions.DjangoModelPermissionsOrAnonReadOnly, )
    authentication_classes = (authentication.SessionAuthentication,)
    queryset = Layer.objects.published()
    serializer_class = LayerDetailSerializer
    lookup_field = 'slug'

layer_detail = LayerDetail.as_view()


class LayerNodesList(NodeList):
    """
    Retrieve list of nodes of the specified layer

    Parameters:

     * `search=<word>`: search <word> in name of nodes of specified layer
     * `limit=<n>`: specify number of items per page (defaults to 40)
    """
    layer = None

    def get_layer(self):
        """ retrieve layer from DB """
        if self.layer:
            return
        try:
            self.layer = Layer.objects.get(slug=self.kwargs['slug'])
        except Layer.DoesNotExist:
            raise Http404(_('Layer not found'))

    def get_queryset(self):
        """ extend parent class queryset by filtering nodes of the specified layer """
        self.get_layer()
        return super(LayerNodesList, self).get_queryset().filter(layer_id=self.layer.id)

    def get_nodes(self, request, *args, **kwargs):
        """ this method might be overridden by other modules (eg: nodeshot.interop.sync) """
        # ListSerializerMixin.list returns a serializer object
        return (self.list(request, *args, **kwargs)).data

    def get(self, request, *args, **kwargs):
        """ Retrieve list of nodes of the specified layer """
        self.get_layer()
        # get nodes of layer
        nodes = self.get_nodes(request, *args, **kwargs)
        return Response(nodes)

    post = Hider()

nodes_list = LayerNodesList.as_view()


class LayerNodesGeoJSONList(LayerNodesList):
    """
    Retrieve list of nodes of the specified layer in GeoJSON format.

    Parameters:

     * `search=<word>`: search <word> in name, slug, description and address of nodes
     * `limit=<n>`: specify number of items per page (show all by default)
    """
    pagination_serializer_class = PaginatedGeojsonNodeListSerializer
    paginate_by_param = 'limit'
    paginate_by = 0
    serializer_class = NodeGeoSerializer

    def get(self, request, *args, **kwargs):
        """ Retrieve list of nodes of the specified layer in GeoJSON format. """
        # overwritten just to tweak the docstring for auto documentation purposes
        return super(LayerNodesGeoJSONList, self).get(request, *args, **kwargs)

nodes_geojson_list = LayerNodesGeoJSONList.as_view()


class LayerGeoJSONList(generics.ListAPIView):
    """
    Retrieve list of layers in GeoJSON format.
    Parameters:

     * `limit=<n>`: specify number of items per page (defaults to 40)
     * `page=<n>`: show page n
    """
    pagination_serializer_class = PaginatedGeojsonLayerListSerializer
    paginate_by_param = 'limit'
    paginate_by = 40
    serializer_class = GeoLayerListSerializer
    queryset = Layer.objects.published().exclude(area__isnull=True)

layers_geojson_list = LayerGeoJSONList.as_view()



# this app is dependant on "net"
from django.conf import settings
if 'nodeshot.networking.net' not in settings.INSTALLED_APPS:
    from nodeshot.core.base.exceptions import DependencyError
    raise DependencyError('nodeshot.networking.services depends on nodeshot.networking.net, which should be in settings.INSTALLED_APPS')


from category import Category
from service import Service
from url import Url
from service_login import ServiceLogin

__all__ = ['Service', 'Category', 'Url', 'ServiceLogin']


from django.db import models
from django.core import validators
from django.core.mail import send_mail
from django.contrib.auth.models import AbstractBaseUser, PermissionsMixin
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth.models import Group

from nodeshot.core.base.utils import now
from ..managers import ProfileManager
from ..settings import settings, EMAIL_CONFIRMATION, REQUIRED_FIELDS as PROFILE_REQUIRED_FIELDS
from ..signals import password_changed

import re


SEX_CHOICES = (
    ('M', _('male')),
    ('F', _('female'))
)


class Profile(AbstractBaseUser, PermissionsMixin):
    """
    User Profile Model
    Contains personal info of a user
    """
    # 254 maximum character for username makes it possible
    username = models.CharField(
        _('username'),
        max_length=254,
        unique=True,
        db_index=True,
        help_text=_('Required. 30 characters or fewer.\
                    Letters, numbers and @/./+/-/_ characters'),
        validators=[
            validators.RegexValidator(
                re.compile('^[\w.@+-]+$'),
                _('Enter a valid username.'),
                'invalid'
            )
        ]
    )
    email = models.EmailField(_('primary email address'), blank=True, unique=True, db_index=True)
    first_name = models.CharField(_('first name'), max_length=30, blank=True)
    last_name = models.CharField(_('last name'), max_length=30, blank=True)

    # added fields
    about = models.TextField(_('about me'), blank=True)
    gender = models.CharField(_('gender'), max_length=1, choices=SEX_CHOICES, blank=True)
    birth_date = models.DateField(_('birth date'), blank=True, null=True)
    address = models.CharField(_('address'), max_length=150, blank=True)
    city = models.CharField(_('city'), max_length=30, blank=True)
    country = models.CharField(_('country'), max_length=30, blank=True)

    is_staff = models.BooleanField(_('staff status'), default=False,
                                   help_text=_('Designates whether the user can log into this admin site.'))
    is_active = models.BooleanField(
        _('active'),
        default=True,
        help_text=_('Designates whether this user should be treated as active.\
                    Unselect this instead of deleting accounts.')
    )
    date_joined = models.DateTimeField(_('date joined'), default=now)

    objects = ProfileManager()

    USERNAME_FIELD = 'username'
    REQUIRED_FIELDS = PROFILE_REQUIRED_FIELDS

    class Meta:
        verbose_name = _('user')
        verbose_name_plural = _('users')
        app_label = 'profiles'

    def __unicode__(self):
        return self.username

    def save(self, *args, **kwargs):
        """
        performs actions on create:
            * ensure instance has usable password
            * set default group
            * keep in sync with email address model
        """
        created = self.pk is None
        sync_emailaddress = kwargs.pop('sync_emailaddress', True)
        # ensure usable password
        if created and self.has_usable_password() is False:
            self.set_password(self.password)
        # save
        super(Profile, self).save(*args, **kwargs)
        # add user to default group
        if created and self.groups.count() < 1:
            # TODO: make default group configurable in settings
            try:
                default_group = Group.objects.get(name='registered')
                self.groups.add(default_group)
            except Group.DoesNotExist:
                pass
        # keep in sync with EmailAddress model
        if (EMAIL_CONFIRMATION and sync_emailaddress and self.email
            and self.email_set.filter(email=self.email).count() < 1
        ):
            self.email_set.add_email(self, email=self.email)
            self.email_set.last().set_as_primary()

    def get_full_name(self):
        """
        Returns the first_name plus the last_name, with a space in between.
        """
        full_name = '%s %s' % (self.first_name, self.last_name)
        return full_name.strip()

    def get_short_name(self):
        "Returns the short name for the user."
        return self.first_name

    def email_user(self, subject, message, from_email=None):
        """
        Sends an email to this User.
        """
        send_mail(subject, message, from_email, [self.email])

    def needs_confirmation(self):
        """
        set is_active to False if needs email confirmation
        """
        if EMAIL_CONFIRMATION:
            self.is_active = False
            self.save()
            return True
        else:
            return False

    def change_password(self, new_password):
        """
        Changes password and sends a signal
        """
        self.set_password(new_password)
        self.save()
        password_changed.send(sender=self.__class__, user=self)

    if 'grappelli' in settings.INSTALLED_APPS:
        @staticmethod
        def autocomplete_search_fields():
            return (
                'username__icontains',
                'first_name__icontains',
                'last_name__icontains',
                'email__icontains'
            )



from django.conf.urls import patterns, url


urlpatterns = patterns('nodeshot.networking.services.views',
    url(r'^services/categories/$', 'service_category_list', name='api_service_category_list'),
    url(r'^services/categories/(?P<pk>[0-9]+)/$', 'service_category_details', name='api_service_category_details'),
    url(r'^services/$', 'service_list', name='api_service_list'),
    url(r'^services/(?P<pk>[0-9]+)/$', 'service_details', name='api_service_details'),
)



"""
utilities for caching
"""
from django.core.cache import cache


def cache_delete_pattern_or_all(pattern):
    # clear only cached pages if supported
    if hasattr(cache, 'delete_pattern'):
        cache.delete_pattern(pattern)
    # otherwise clear the entire cache
    else:
        cache.clear()


def cache_by_group(view_instance, view_method, request, args, kwargs):
    """
    Cache view response by media type and user group.
    The cache_key is constructed this way: "{view_name:path.group.media_type}"
    EG: "MenuList:/api/v1/menu/.public.application/json"
    Possible groups are:
        * public
        * superuser
        * the rest are retrieved from DB (registered, community, trusted are the default ones)
    """
    if request.user.is_anonymous():
        group = 'public'
    elif request.user.is_superuser:
        group = 'superuser'
    else:
        try:
            group = request.user.groups.all().order_by('-id').first().name
        except IndexError:
            group = 'public'

    key = '%s:%s.%s.%s' % (
        view_instance.__class__.__name__,
        request.META['PATH_INFO'],
        group,
        request.accepted_media_type
    )

    return key



from __future__ import absolute_import

import requests
from xml.dom import minidom
from dateutil import parser as DateParser

from django.template.defaultfilters import slugify
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth import get_user_model
User = get_user_model()

from nodeshot.core.base.utils import pause_disconnectable_signals, resume_disconnectable_signals
from nodeshot.core.nodes.models import Node, Status


__all__ = [
    # classes
    'BaseSynchronizer',
    'XmlSynchronizer',
    'GenericGisSynchronizer',
    # mixins
    'HttpRetrieverMixin',
    'XMLParserMixin',
]


class BaseSynchronizer(object):
    """
    Base Synchronizer
    Provides methods for:
        * config validation
        * django validation
        * executing actions before or after specific events
        * retrieve data
        * extract data from imported source
        * save data into DB
        * log messages with different levels of verbosity
    """
    SCHEMA = None

    def __init__(self, layer, *args, **kwargs):
        """
        :layer models.Model: instance of Layer we want to convert
        """
        self.layer = layer
        self.verbosity = kwargs.get('verbosity', 1)
        self.load_config()

    def load_config(self, config=None):
        self.config = config or self.layer.external.config
        self.verify_ssl = self.config.get('verify_ssl', True)
        # ensure correct boolean
        self.verify_ssl = self.verify_ssl is True or self.verify_ssl == 'True'

    def clean(self):
        """ complex ad hoc validation here, will be executed before the external layer is saved """
        pass

    def after_external_layer_saved(self, *args, **kwargs):
        """ anything that should be executed after the external layer is saved goes here """
        pass

    def before_start(self, *args, **kwargs):
        """ anything that should be executed before the import starts goes here """
        pass

    def after_complete(self, *args, **kwargs):
        """ anything that should be executed after the import is complete goes here """
        pass

    def process(self):
        """
        This is the method that does everything automatically (at least attempts to).

        Steps:
            0. Call "before_start" method (which might be implemented by children classes)
            1. Retrieve data from external source
            2. Parse the data
            3. Save the data locally
            4. Call "after_complete" method (which might be implemented by children classes)
        """
        self.before_start()
        self.retrieve_data()
        self.parse()

        # TRICK: disable new_nodes_allowed_for_layer validation
        try:
            Node._additional_validation.remove('new_nodes_allowed_for_layer')
        except ValueError as e:
            print "WARNING! got exception: %s" % e
        # avoid sending zillions of notifications
        pause_disconnectable_signals()

        self.save()

        # Re-enable new_nodes_allowed_for_layer validation
        try:
            Node._additional_validation.insert(0, 'new_nodes_allowed_for_layer')
        except ValueError as e:
            print "WARNING! got exception: %s" % e
        # reconnect signals
        resume_disconnectable_signals()

        self.after_complete()

        # return message as a list because more than one messages might be returned
        return [self.message]

    def retrieve_data(self):
        """ retrieve data """
        raise NotImplementedError("BaseSynchronizer child class does not implement a retrieve_data method")

    def parse(self):
        """ parse data """
        raise NotImplementedError("BaseSynchronizer child class does not implement a parse method")

    def save(self):
        """ save data into DB """
        raise NotImplementedError("BaseSynchronizer child class does not implement a save method")

    def verbose(self, message):
        if self.verbosity >= 2:
            print(message)


class HttpRetrieverMixin(object):
    """ Retrieve external data through HTTP """

    def retrieve_data(self):
        """ retrieve data from an HTTP URL """
        # shortcuts for readability
        url = self.config.get('url')

        # do HTTP request and store content
        self.data = requests.get(url, verify=self.verify_ssl).content


class XMLParserMixin(object):
    """ XML Parsing utility methods """

    def parse(self):
        """ parse data """
        self.parsed_data = minidom.parseString(self.data)

    @staticmethod
    def get_text(item, tag, default=False):
        """ returns text content of an xml tag """
        try:
            xmlnode = item.getElementsByTagName(tag)[0].firstChild
        except IndexError as e:
            if default is not False:
                return default
            else:
                raise IndexError(e)

        if xmlnode is not None:
            return unicode(xmlnode.nodeValue)
        # empty tag
        else:
            return ''


class GenericGisSynchronizer(HttpRetrieverMixin, BaseSynchronizer):
    """
    Base Synchronizer for GIS formats like geojson, georss, kml, ecc

    It does not supports all the formats, rather it provides an easy way to add support for each format.

    You must implement a "parse_item" method to support different formats.
    """
    SCHEMA = [
        {
            'name': 'url',
            'class': 'CharField',
            'kwargs': {
                'max_length': 128,
                'help_text': _('URL containing geographical data')
            }
        },
        {
            'name': 'verify_ssl',
            'class': 'BooleanField',
            'kwargs': {
                'default': True,
                'help_text': _('Wether the SSL certificates of the external services used should be verified or not')
            }
        },
        {
            'name': 'default_status',
            'class': 'CharField',
            'kwargs': {
                'blank': True,
                'max_length': 255,
                'help_text': _('Status for imported nodes, leave blank to use the default one')
            }
        },
        {
            'name': 'field_name',
            'class': 'CharField',
            'kwargs': {
                'max_length': 64,
                'default': 'name',
                'verbose_name': _('name field'),
                'help_text': _('corresponding name field on external source')
            }
        },
        {
            'name': 'field_status',
            'class': 'CharField',
            'kwargs': {
                'max_length': 64,
                'default': 'status',
                'verbose_name': _('status field'),
                'help_text': _('corresponding status field on external source')
            }
        },
        {
            'name': 'field_description',
            'class': 'CharField',
            'kwargs': {
                'max_length': 64,
                'default': 'description',
                'verbose_name': _('description field'),
                'help_text': _('corresponding description field on external source')
            }
        },
        {
            'name': 'field_address',
            'class': 'CharField',
            'kwargs': {
                'max_length': 64,
                'default': 'address',
                'verbose_name': _('address field'),
                'help_text': _('corresponding address field on external source')
            }
        },
        {
            'name': 'field_is_published',
            'class': 'CharField',
            'kwargs': {
                'max_length': 64,
                'default': 'is_published',
                'verbose_name': _('is_published field'),
                'help_text': _('corresponding is_published field on external source')
            }
        },
        {
            'name': 'field_user',
            'class': 'CharField',
            'kwargs': {
                'max_length': 64,
                'default': 'user',
                'verbose_name': _('user field'),
                'help_text': _('corresponding user field on external source')
            }
        },
        {
            'name': 'field_elev',
            'class': 'CharField',
            'kwargs': {
                'max_length': 64,
                'default': 'elev',
                'verbose_name': _('elev field'),
                'help_text': _('corresponding elev field on external source')
            }
        },
        {
            'name': 'field_notes',
            'class': 'CharField',
            'kwargs': {
                'max_length': 64,
                'default': 'notes',
                'verbose_name': _('notes field'),
                'help_text': _('corresponding notes field on external source')
            }
        },
        {
            'name': 'field_added',
            'class': 'CharField',
            'kwargs': {
                'max_length': 64,
                'default': 'added',
                'verbose_name': _('added field'),
                'help_text': _('corresponding added field on external source')
            }
        },
        {
            'name': 'field_updated',
            'class': 'CharField',
            'kwargs': {
                'max_length': 64,
                'default': 'updated',
                'verbose_name': _('updated field'),
                'help_text': _('corresponding updated field on external source')
            }
        }
    ]

    def __init__(self, layer, *args, **kwargs):
        super(GenericGisSynchronizer, self).__init__(layer, *args, **kwargs)
        # init empty dict
        self.field_mapping = {}
        # include all keys which start with 'field_'
        for key, value in layer.external.config.items():
            if key.startswith('field_'):
                # replace 'field_' prefix
                key = key.replace('field_', '')
                self.field_mapping[key] = value

    def parse_item(self, item):
        """
        override this method according to the format you want to support.

        Should return a dictionary with the following structure:

        result = {
            "name": "string required",
            "status": "string or None",
            "address": "string or empty string",
            "is_published": "boolean",
            "user": "username or None",
            "geometry": "GEOSGeometry object",
            "elev": "float or none",
            "description": "string or empty string",
            "notes": "string or empty string",
            "added", "string date representation",
            "updated", "string date representation",
        }
        """
        raise NotImplementedError("Not Implemented")

    def _convert_item(self, item):
        """
        take a parsed item as input and returns a python dictionary
        the keys will be saved into the Node model
        either in their respective fields or in the hstore "data" field

        :param item: object representing parsed item
        """
        item = self.parse_item(item)

        # name is required
        if not item['name']:
            raise Exception('Expected property %s not found in item %s.' % (self.keys['name'], item))

        if not item['status']:
            item['status'] = self.default_status

        # get status or get default status or None
        try:
            item['status'] = Status.objects.get(slug__iexact=item['status'])
        except Status.DoesNotExist:
            try:
                item['status'] = Status.objects.filter(is_default=True)[0]
            except IndexError:
                item['status'] = None

        # slugify slug
        item['slug'] = slugify(item['name'])

        if not item['address']:
            item['address'] = ''

        if not item['is_published']:
            item['is_published'] = ''

        # get user or None
        try:
            item['user'] = User.objects.get(username=item['user'])
        except User.DoesNotExist:
            item['user'] = None

        if not item['elev']:
            item['elev'] = None

        if not item['description']:
            item['description'] = ''

        if not item['notes']:
            item['notes'] = ''

        # convert dates to python datetime
        try:
            item['added'] = DateParser.parse(item['added'])
        except Exception as e:
            print "Exception while parsing 'added' date: %s" % e
        try:
            item['updated'] = DateParser.parse(item['updated'])
        except Exception as e:
            print "Exception while parsing 'updated' date: %s" % e

        result = {
            "name": item['name'],
            "slug": item['slug'],
            "status": item['status'],
            "address": item['address'],
            "is_published": item['is_published'],
            "user": item['user'],
            "geometry": item['geometry'],
            "elev": item['elev'],
            "description": item['description'],
            "notes": item['notes'],
            "added": item['added'],
            "updated": item['updated'],
            "data": {}
        }

        # ensure all additional data items are strings
        for key, value in item['data'].items():
            result["data"][key] = value

        return result

    def key_mapping(self):
        key_map = self.field_mapping
        self.keys = {
            "name": key_map.get('name', 'name'),
            "status": key_map.get('status', 'status'),
            "description": key_map.get('description', 'description'),
            "address": key_map.get('address', 'address'),
            "is_published": key_map.get('is_published', 'is_published'),
            "user": key_map.get('user', 'user'),
            "elev": key_map.get('elev', 'elev'),
            "notes": key_map.get('notes', 'notes'),
            "added": key_map.get('added', 'added'),
            "updated": key_map.get('updated', 'updated'),
        }
        self.default_status = self.config.get('default_status', '')

    def save(self):
        """
        save data into DB:

         1. save new (missing) data
         2. update only when needed
         3. delete old data
         4. generate report that will be printed

        constraints:
         * ensure new nodes do not take a name/slug which is already used
         * validate through django before saving
         * use good defaults
        """
        self.key_mapping()
        # retrieve all items
        items = self.parsed_data

        # init empty lists
        added_nodes = []
        changed_nodes = []
        unmodified_nodes = []

        # retrieve a list of all the slugs of this layer
        layer_nodes_slug_list = Node.objects.filter(layer=self.layer).values_list('slug', flat=True)
        # keep a list of all the nodes of other layers
        other_layers_slug_list = Node.objects.exclude(layer=self.layer).values_list('slug', flat=True)
        # init empty list of slug of external nodes that will be needed to perform delete operations
        processed_slug_list = []
        deleted_nodes_count = 0

        # loop over every item
        for item in items:

            item = self._convert_item(item)

            number = 1
            original_name = item['name']
            needed_different_name = False

            while True:
                # items might have the same name... so we add a number..
                if item['slug'] in processed_slug_list or item['slug'] in other_layers_slug_list:
                    needed_different_name = True
                    number = number + 1
                    item['name'] = "%s - %d" % (original_name, number)
                    item['slug'] = slugify(item['name'])
                else:
                    if needed_different_name:
                        self.verbose('needed a different name for %s, trying "%s"' % (original_name, item['name']))
                    break

            # default values
            added = False
            changed = False

            try:
                # edit existing node
                node = Node.objects.get(slug=item['slug'], layer=self.layer)
            except Node.DoesNotExist:
                # add a new node
                node = Node()
                node.layer = self.layer
                added = True

            # loop over fields and store data only if necessary
            for field in Node._meta.fields:
                # geometry is a special case, skip
                if field.name == 'geometry':
                    continue
                # skip if field is not present in values
                if field.name not in item.keys():
                    continue
                # shortcut for value
                value = item[field.name]
                # if value is different than what we have
                if getattr(node, field.name) != value and value is not None:
                    # set value
                    setattr(node, field.name, value)
                    # indicates that a DB query is necessary
                    changed = True

            if added is True or (node.geometry.equals(item['geometry']) is False\
                                 and node.geometry.equals_exact(item['geometry']) is False):
                node.geometry = item['geometry']
                changed = True

            node.data = node.data or {}

            # store any additional key/value in HStore data field
            for key, value in item['data'].items():
                if node.data[key] != value:
                    node.data[key] = value
                    changed = True

            # perform save or update only if necessary
            if added or changed:
                try:
                    node.full_clean()
                    if node.added is not None and node.updated is not None:
                        node.save(auto_update=False)
                    else:
                        node.save()
                except Exception as e:
                    raise Exception('error while processing "%s": %s' % (node.name, e))

            if added:
                added_nodes.append(node)
                self.verbose('new node saved with name "%s"' % node.name)
            elif changed:
                changed_nodes.append(node)
                self.verbose('node "%s" updated' % node.name)
            else:
                unmodified_nodes.append(node)
                self.verbose('node "%s" unmodified' % node.name)

            # fill node list container
            processed_slug_list.append(node.slug)

        # delete old nodes
        for local_node in layer_nodes_slug_list:
            # if local node not found in external nodes
            if local_node not in processed_slug_list:
                # retrieve from DB and delete
                node = Node.objects.get(slug=local_node)
                # store node name to print it later
                node_name = node.name
                node.delete()
                # then increment count that will be included in message
                deleted_nodes_count = deleted_nodes_count + 1
                self.verbose('node "%s" deleted' % node_name)

        # message that will be returned
        self.message = """
            %s nodes added
            %s nodes changed
            %s nodes deleted
            %s nodes unmodified
            %s total external records processed
            %s total local nodes for this layer
        """ % (
            len(added_nodes),
            len(changed_nodes),
            deleted_nodes_count,
            len(unmodified_nodes),
            len(items),
            Node.objects.filter(layer=self.layer).count()
        )


class XmlSynchronizer(HttpRetrieverMixin, XMLParserMixin, BaseSynchronizer):
    """ XML HTTP syncrhonizer """



import sys
import re
import json

# ------- IFCONFIG TO JSON CONVERSION ------- #
# credits:
# https://gist.github.com/snbartell/1586034

def _extract(ifconfig_output):
    mo = re.search(r'^(?P<interface>\w+|\w+:\d+)\s+' +
                     r'Link encap:(?P<link_encap>\S+)\s+' +
                     r'(HWaddr\s+(?P<hardware_address>\S+))?' +
                     r'(\s+inet addr:(?P<ip_address>\S+))?' +
                     r'(\s+inet6 addr:(?P<ipv6_address>\S+)Scope:Global)?' +
                     r'(\s+Bcast:(?P<broadcast_address>\S+)\s+)?' +
                     r'(Mask:(?P<net_mask>\S+)\s+)?',
                     ifconfig_output, re.MULTILINE|re.IGNORECASE )
    if mo:
        info = mo.groupdict('')
        info['running'] = False
        info['up'] = False
        info['multicast'] = False
        info['broadcast'] = False
        if 'RUNNING' in ifconfig_output:
            info['running'] = True
        if 'UP' in ifconfig_output:
            info['up'] = True
        if 'BROADCAST' in ifconfig_output:
            info['broadcast'] = True
        if 'MULTICAST' in ifconfig_output:
            info['multicast'] = True
        return info
    return {}
 

def ifconfig_to_dict(ifconfig):
    interfaces = [ _extract(interface) for interface in ifconfig.split('\n\n') if interface.strip() ]
    return interfaces

def ifconfig_to_json(ifconfig, indent=4):
    interfaces = ifconfig_to_dict(ifconfig)
    return json.dumps(interfaces, indent=indent)


from django.conf.urls import patterns, include, url

from .settings import settings, API_PREFIX, API_APPS_ENABLED


urlpatterns = patterns('nodeshot.core.api.views',
    url(r'^%s$' % API_PREFIX, 'root_endpoint', name='api_root_endpoint'),
)

if 'rest_framework_swagger' in settings.INSTALLED_APPS:
    urlpatterns += patterns('',
        url(r'^%sdocs/' % API_PREFIX, include('rest_framework_swagger.urls'))
    )

# loop over all the strings listed in API_APPS_ENABLED
for app_path in API_APPS_ENABLED:

    # enable only API modules listed in INSTALLED_APPS
    if app_path not in settings.INSTALLED_APPS:
        # fail silently
        continue

    # determine import path for url patterns
    module_path = '%s.urls' % app_path

    urlpatterns += patterns('',
        url(r'^%s' % API_PREFIX, include(module_path))
    )



from django.conf.urls import patterns, url


urlpatterns = patterns('nodeshot.networking.net.views',
    url(r'^devices/$', 'device_list', name='api_device_list'),
    url(r'^devices/(?P<pk>[0-9]+)/$', 'device_details', name='api_device_details'),
    url(r'^nodes/(?P<slug>[-\w]+)/devices/$', 'node_device_list', name='api_node_devices'),
    
    # interfaces
    url(r'^devices/(?P<pk>[0-9]+)/ethernet/$', 'device_ethernet_list', name='api_device_ethernet'),
    url(r'^ethernet/(?P<pk>[0-9]+)/$', 'ethernet_details', name='api_ethernet_details'),
    url(r'^devices/(?P<pk>[0-9]+)/wireless/$', 'device_wireless_list', name='api_device_wireless'),
    url(r'^wireless/(?P<pk>[0-9]+)/$', 'wireless_details', name='api_wireless_details'),
    url(r'^devices/(?P<pk>[0-9]+)/bridge/$', 'device_bridge_list', name='api_device_bridge'),
    url(r'^bridge/(?P<pk>[0-9]+)/$', 'bridge_details', name='api_bridge_details'),
    url(r'^devices/(?P<pk>[0-9]+)/tunnel/$', 'device_tunnel_list', name='api_device_tunnel'),
    url(r'^tunnel/(?P<pk>[0-9]+)/$', 'tunnel_details', name='api_tunnel_details'),
    url(r'^devices/(?P<pk>[0-9]+)/vlan/$', 'device_vlan_list', name='api_device_vlan'),
    url(r'^vlan/(?P<pk>[0-9]+)/$', 'vlan_details', name='api_vlan_details'),
    
    # ip
    url(r'^interfaces/(?P<pk>[0-9]+)/ip/$', 'interface_ip_list', name='api_interface_ip'),
    url(r'^ip/(?P<pk>[0-9]+)/$', 'ip_details', name='api_ip_details'),
)


from django.conf.urls import patterns, url


urlpatterns = patterns('nodeshot.core.layers.views',
    url(r'^layers/$', 'layer_list', name='api_layer_list'),
    url(r'^layers/(?P<slug>[-\w]+)/$', 'layer_detail', name='api_layer_detail'),
    url(r'^layers/(?P<slug>[-\w]+)/nodes/$', 'nodes_list', name='api_layer_nodes_list'),
    url(r'^layers/(?P<slug>[-\w]+)/nodes.geojson$', 'nodes_geojson_list', name='api_layer_nodes_geojson'),
    url(r'^layers.geojson$', 'layers_geojson_list', name='api_layer_geojson'),
)



from django.contrib import admin
from django.contrib import messages
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth import get_user_model
User = get_user_model()

from nodeshot.core.base.admin import BaseAdmin
from nodeshot.core.layers.models import Layer

from .models import Inward, Outward
from .settings import settings, OUTWARD_HTML
from .tasks import send_outward_mails


class InwardAdmin(BaseAdmin):
    list_display = ('from_email', 'from_name', 'to', 'status', 'added', 'updated')
    search_fields = ('from_email', 'from_name')
    # define the autocomplete_lookup_fields
    if 'grappelli' in settings.INSTALLED_APPS:
        autocomplete_lookup_fields = {
            'generic': [['content_type', 'object_id']],
        }


class OutwardAdmin(BaseAdmin):
    list_display = ('subject', 'status', 'is_scheduled', 'added', 'updated')
    list_filter = ('status', 'is_scheduled')
    filter_horizontal = ['layers', 'users']
    search_fields = ('subject',)
    actions = ['send']
    change_form_template = 'admin/outward_change_form.html'

    def send(self, request, queryset):
        """
        Send action available in change outward list
        """
        send_outward_mails.delay(queryset)
        # show message in the admin
        messages.info(request, _('Message sent successfully'), fail_silently=True)
    send.short_description = _('Send selected messages')

    def formfield_for_manytomany(self, db_field, request, **kwargs):
        if db_field.name == 'layers':
            kwargs['queryset'] = Layer.objects.filter(is_external=False)
        if db_field.name == 'users':
            kwargs['queryset'] = User.objects.filter(is_active=True)
        return super(OutwardAdmin, self).formfield_for_manytomany(db_field, request, **kwargs)

    # Enable TinyMCE HTML Editor according to settings, defaults to True
    if OUTWARD_HTML:
        # enable editor for "description" only
        html_editor_fields = ['message']

admin.site.register(Inward, InwardAdmin)
admin.site.register(Outward, OutwardAdmin)



from django.http import Http404

from rest_framework import generics, permissions, authentication
from rest_framework.response import Response

from .models import *
from .serializers import *


class NotificationList(generics.ListAPIView):
    """
    Retrieve a list of notifications of the current user.

    **Available variations through querystring parameters:**

     * `action=unread`: default behaviour, retrieve unread notifications and mark them as read
     * `action=unread&read=false`: retrieve unread notifications without marking them as read
     * `action=count`: retrieve count of unread notifications without marking them as read
     * `action=all`: retrieve all notifications with pagination
        * `limit=<n>`: specify number of items per page (defaults to 30)
        * `limit=0`: turns off pagination
    """
    authentication_classes = (authentication.SessionAuthentication,)
    permission_classes = (permissions.IsAuthenticated,)
    paginate_by = 30
    paginate_by_param = 'limit'
    serializer_class = NotificationSerializer
    pagination_serializer_class = PaginatedNotificationSerializer
    queryset = Notification.objects.select_related('from_user')

    def get_queryset(self):
        """ filter only notifications of current user """
        return self.queryset.filter(to_user=self.request.user)

    def get(self, request, format=None):
        """ get HTTP method """
        action = request.QUERY_PARAMS.get('action', 'unread')
        # action can be only "unread" (default), "count" and "all"
        action = action if action == 'count' or action == 'all' else 'unread'
        # mark as read parameter, defaults to true
        mark_as_read = request.QUERY_PARAMS.get('read', 'true') == 'true'
        # queryset
        notifications = self.get_queryset().filter(to_user=request.user)
        # pass to specific action
        return getattr(self, 'get_%s' % action)(request, notifications, mark_as_read)

    def get_unread(self, request, notifications, mark_as_read):
        """ return unread notifications and mark as read (unless read=false param is passed)"""
        notifications = notifications.filter(is_read=False)
        data = UnreadNotificationSerializer(notifications, many=True).data
        # if True mark retrieve unread notifications as read (default behaviour)
        if mark_as_read:
            notifications.update(is_read=True)
        return Response(data)

    def get_count(self, request, notifications, mark_as_read=False):
        """ return count of unread notification """
        data = {'count': notifications.filter(is_read=False).count()}
        return Response(data)

    def get_all(self, request, notifications, mark_as_read=False):
        """ return all notifications with pagination """
        notifications = notifications
        return self.list(request, notifications)

notification_list = NotificationList.as_view()


class NotificationDetail(generics.RetrieveAPIView):
    """
    Retrieve specific notification of current user.
    """
    authentication_classes = (authentication.SessionAuthentication,)
    permission_classes = (permissions.IsAuthenticated,)
    serializer_class = NotificationSerializer
    queryset = Notification.objects.select_related('from_user')

    def get_queryset(self):
        """ filter only notifications of current user """
        return self.queryset.filter(to_user=self.request.user)

notification_detail = NotificationDetail.as_view()


# ------ User Notification Settings ------ #


class EmailNotificationSettings(generics.RetrieveUpdateAPIView):
    """
    Retrieve Email Notification settings.

    ### PUT & PATCH

    Edit Email Notification settings.
    """
    authentication_classes = (authentication.SessionAuthentication,)
    permission_classes = (permissions.IsAuthenticated,)
    serializer_class = EmailNotificationSettingsSerializer
    model = UserEmailNotificationSettings

    def get_queryset(self):
        return self.model.objects.get(user_id=self.request.user.id)

    def get_object(self, queryset=None):
        """ get privacy settings of current user """
        try:
            obj = self.get_queryset()
        except self.model.DoesNotExist:
            raise Http404()
        self.check_object_permissions(self.request, obj)
        return obj

notification_email_settings = EmailNotificationSettings.as_view()


class WebNotificationSettings(EmailNotificationSettings):
    """
    Retrieve Web Notification settings.

    ### PUT & PATCH

    Edit Web Notification settings.
    """
    authentication_classes = (authentication.SessionAuthentication,)
    permission_classes = (permissions.IsAuthenticated,)
    serializer_class = WebNotificationSettingsSerializer
    model = UserWebNotificationSettings

notification_web_settings = WebNotificationSettings.as_view()



from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.conf import settings
from django.core.exceptions import ValidationError

from nodeshot.core.base.models import BaseDate

from .base import UpdateCountsMixin


class Vote(UpdateCountsMixin, BaseDate):
    """
    Vote model
    Like or dislike feature
    """

    VOTING_CHOICES = (
        (1, 'Like'),
        (-1, 'Dislike'),
    )

    node = models.ForeignKey('nodes.Node')
    user = models.ForeignKey(settings.AUTH_USER_MODEL)
    # TODO: this should also be called "value" instead of "vote"
    vote = models.IntegerField(choices=VOTING_CHOICES)

    class Meta:
        app_label = 'participation'

    def __unicode__(self):
        if self.pk:
            return _('vote #%d for node %s') % (self.pk, self.node.name)
        else:
            return _('vote')

    def save(self, *args, **kwargs):
        """
        ensure users cannot vote the same node multiple times
        but let users change their votes
        """
        if not self.pk:
            old_votes = Vote.objects.filter(user=self.user, node=self.node)
            for old_vote in old_votes:
                old_vote.delete()
        super(Vote, self).save(*args, **kwargs)

    def update_count(self):
        """ updates likes and dislikes count """
        node_rating_count = self.node.rating_count
        node_rating_count.likes = self.node.vote_set.filter(vote=1).count()
        node_rating_count.dislikes = self.node.vote_set.filter(vote=-1).count()
        node_rating_count.save()

    def clean(self, *args, **kwargs):
        """
        Check if votes can be inserted for parent node or parent layer
        """
        if not self.pk:
            # ensure voting for this node is allowed
            if self.node.participation_settings.voting_allowed is not True:
                raise ValidationError("Voting not allowed for this node")

            if 'nodeshot.core.layers' in settings.INSTALLED_APPS:
                layer = self.node.layer

                # ensure voting for this layer is allowed
                if layer.participation_settings.voting_allowed is not True:
                    raise ValidationError("Voting not allowed for this layer")



from django.core.management.base import BaseCommand

from nodeshot.community.notifications.models import Notification
from nodeshot.core.base.utils import ago

from ...settings import settings, DELETE_OLD


class Command(BaseCommand):
    help = "Delete notifications older than DELETE_OLD"

    def retrieve_old_notifications(self):
        """
        Retrieve notifications older than X days, where X is specified in settings
        """

        date = ago(days=DELETE_OLD)

        return Notification.objects.filter(added__lte=date)

    def output(self, message):
        self.stdout.write('%s\n\r' % message)

    def handle(self, *args, **options):
        """ Purge notifications """
        # retrieve layers
        notifications = self.retrieve_old_notifications()
        count = len(notifications)

        if count > 0:
            self.output('found %d notifications to purge...' % count)
            notifications.delete()
            self.output('%d notifications deleted successfully.' % count)
        else:
            self.output('there are no old notifications to purge')



from django.contrib import admin
from nodeshot.core.base.admin import BaseAdmin

from .models import *
from .settings import settings


class NotificationAdmin(BaseAdmin):
    list_display = ('to_user', 'type', 'text', 'is_read', 'added', 'updated')
    list_filter = ('type', 'is_read', 'added')

    raw_id_fields = ('from_user', 'to_user')
    autocomplete_lookup_fields = {
        'fk': ['from_user', 'to_user'],
    }


admin.site.register(Notification, NotificationAdmin)


if 'nodeshot.community.profiles' in settings.INSTALLED_APPS:

    class UserWebNotificationSettingsInline(admin.StackedInline):
        model = UserWebNotificationSettings
        extra = 1

    class UserEmailNotificationSettingsInline(admin.StackedInline):
        model = UserEmailNotificationSettings
        extra = 1

    from nodeshot.community.profiles.admin import UserAdmin

    additional_inlines = [UserWebNotificationSettingsInline, UserEmailNotificationSettingsInline]

    UserAdmin.inlines = UserAdmin.inlines + additional_inlines



from rest_framework import permissions
from .models import Profile


__all__ = [
    'IsProfileOwner',
    'IsNotAuthenticated'
]


class IsProfileOwner(permissions.IsAuthenticated):
    """
    Restrict edit to owners only
    """
    def has_object_permission(self, request, view, obj=None):
        # in edit request restrict permission to profile owner only
        if (request.method in ['PUT', 'PATCH']) and obj is not None:
            model = obj.__class__.__name__

            user_id = obj.id

            # in case of social link view
            if model == 'SocialLink':
                user_id = obj.user.id

            return request.user.id == user_id
        else:
            return True

    def has_permission(self, request, view):
        """ applies to social-link-list """
        if request.method == 'POST':
            user = Profile.objects.only('id', 'username').get(username=view.kwargs['username'])
            return request.user.id == user.id

        return True


class IsNotAuthenticated(permissions.IsAuthenticated):
    """
    Restrict access only to unauthenticated users.
    """
    def has_permission(self, request, view, obj=None):
        if request.user and request.user.is_authenticated():
            return False
        else:
            return True



from djcelery_email.backends import CeleryEmailBackend

from .utils import write


class CeleryInfluxDbEmailBackend(CeleryEmailBackend):
    def send_messages(self, email_messages):
        result_tasks = super(CeleryInfluxDbEmailBackend, self).send_messages(email_messages)
        # collect email stats
        write(name='emails_sent', values={'value': len(email_messages)})
        return result_tasks



from django.db import models
from django.utils.translation import ugettext_lazy as _

from nodeshot.core.base.models import BaseDate
from . import ImageMixin, Manufacturer


class DeviceModel(BaseDate, ImageMixin):
    """
    Device Type Model
    Eg.: Nanostation M5, Rocket M2, ecc.
    """
    manufacturer = models.ForeignKey(Manufacturer)
    name = models.CharField(_('name'), max_length=50, unique=True)
    image = models.ImageField(_('image'), upload_to='devices/images/', blank=True)
    datasheet = models.FileField(_('datasheet'), upload_to='devices/datasheets/', blank=True)
    cpu = models.CharField(_('CPU'), max_length=255, blank=True)
    ram = models.IntegerField(_('RAM'), blank=True, help_text=_('bytes'))
    
    def __unicode__(self):
        return self.name
    
    class Meta:
        app_label= 'hardware'
        db_table = 'hardware_device_model'
        verbose_name = _('Device Model')
        verbose_name_plural = _('Device Models')



from django.db import models
from django.utils.translation import ugettext_lazy as _

from nodeshot.core.layers.models import Layer


class LayerParticipationSettings(models.Model):
    """
    Layer settings regarding participation
    """
    layer = models.OneToOneField(Layer, related_name='layer_participation_settings')
    # settings
    voting_allowed = models.BooleanField(_('voting allowed?'), default=True)
    rating_allowed = models.BooleanField(_('rating allowed?'), default=True)
    comments_allowed = models.BooleanField(_('comments allowed?'), default=True)

    def __unicode__(self):
        return self.layer.name

    class Meta:
        app_label = 'participation'
        db_table = 'participation_layer_settings'
        verbose_name_plural = "participation_layer_settings"



"""
Dependencies:
    * nodeshot.core.nodes
    * nodeshot.networking.net
    * django_hstore
"""

from nodeshot.core.base.utils import check_dependencies

check_dependencies(
    dependencies=[
        'nodeshot.core.nodes',
        'nodeshot.networking.net',
        'django_hstore'
    ],
    module='nodeshot.networking.connector'
)


from .device_connector import DeviceConnector


__all__ = ['DeviceConnector']


from django.contrib.gis import admin
from django.conf import settings

from nodeshot.core.base.admin import BaseTabularInline

from .models import Comment, Vote, Rating, NodeParticipationSettings


admin.site.register(Rating)
admin.site.register(Comment)
admin.site.register(Vote)


# ------ EXTEND LAYER ADMIN ------ #

if 'nodeshot.core.layers' in settings.INSTALLED_APPS:

    from nodeshot.core.layers.admin import LayerAdmin
    from .models import LayerParticipationSettings

    class LayerSettingsInline(admin.TabularInline):
        model = LayerParticipationSettings
        extra = 1

    LayerAdmin.inlines.append(LayerSettingsInline)


# ------ EXTEND NODE ADMIN ------ #

from nodeshot.core.nodes.admin import NodeAdmin


class NodeSettingsInline(admin.TabularInline):
    model = NodeParticipationSettings
    extra = 1


class CommentInline(BaseTabularInline):
    model = Comment
    extra = 1


class RatingInline(BaseTabularInline):
    model = Rating
    extra = 1


class VoteInline(BaseTabularInline):
    model = Vote
    extra = 1


additional_inlines = [NodeSettingsInline]

if settings.DEBUG:
    additional_inlines += [CommentInline, RatingInline, VoteInline]

NodeAdmin.inlines = NodeAdmin.inlines + additional_inlines
